
from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .cloud_attachment_gcp_service_account_config import CloudAttachmentGcpServiceAccountConfig
from .cloud_attachment_gcp_user_account_config import CloudAttachmentGcpUserAccountConfig


@JsonMap(
    {
        "gcp_account_type": "gcpAccountType",
        "service_account": "serviceAccount",
        "user_account": "userAccount",
    }
)
class CloudAttachmentGcpSecretsConfig(BaseModel):
    """CloudAttachmentGcpSecretsConfig

    :param gcp_account_type: gcp_account_type
    :type gcp_account_type: str
    :param service_account: service_account, defaults to None
    :type service_account: CloudAttachmentGcpServiceAccountConfig, optional
    :param user_account: user_account, defaults to None
    :type user_account: CloudAttachmentGcpUserAccountConfig, optional
    """

    def __init__(
        self,
        gcp_account_type: str,
        service_account: CloudAttachmentGcpServiceAccountConfig = SENTINEL,
        user_account: CloudAttachmentGcpUserAccountConfig = SENTINEL,
        **kwargs,
    ):
        """CloudAttachmentGcpSecretsConfig

        :param gcp_account_type: gcp_account_type
        :type gcp_account_type: str
        :param service_account: service_account, defaults to None
        :type service_account: CloudAttachmentGcpServiceAccountConfig, optional
        :param user_account: user_account, defaults to None
        :type user_account: CloudAttachmentGcpUserAccountConfig, optional
        """
        self.gcp_account_type = gcp_account_type
        if service_account is not SENTINEL:
            self.service_account = self._define_object(
                service_account, CloudAttachmentGcpServiceAccountConfig
            )
        if user_account is not SENTINEL:
            self.user_account = self._define_object(
                user_account, CloudAttachmentGcpUserAccountConfig
            )
        self._kwargs = kwargs




from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .sftp_get_options import SftpGetOptions
from .sftp_send_options import SftpSendOptions
from .sftp_settings import SftpSettings
from .shared_communication_channel import SharedCommunicationChannel


class SftpCommunicationOptionsCommunicationSetting(Enum):
    """An enumeration representing different categories.

    :cvar DEFAULT: "default"
    :vartype DEFAULT: str
    :cvar CUSTOM: "custom"
    :vartype CUSTOM: str
    :cvar COMPONENT: "component"
    :vartype COMPONENT: str
    """

    DEFAULT = "default"
    CUSTOM = "custom"
    COMPONENT = "component"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                SftpCommunicationOptionsCommunicationSetting._member_map_.values(),
            )
        )


@JsonMap(
    {
        "communication_setting": "CommunicationSetting",
        "sftp_get_options": "SFTPGetOptions",
        "sftp_send_options": "SFTPSendOptions",
        "sftp_settings": "SFTPSettings",
        "shared_communication_channel": "SharedCommunicationChannel",
    }
)
class SftpCommunicationOptions(BaseModel):
    """SftpCommunicationOptions

    :param communication_setting: communication_setting, defaults to None
    :type communication_setting: SftpCommunicationOptionsCommunicationSetting, optional
    :param sftp_get_options: sftp_get_options, defaults to None
    :type sftp_get_options: SftpGetOptions, optional
    :param sftp_send_options: sftp_send_options, defaults to None
    :type sftp_send_options: SftpSendOptions, optional
    :param sftp_settings: sftp_settings, defaults to None
    :type sftp_settings: SftpSettings, optional
    :param shared_communication_channel: shared_communication_channel, defaults to None
    :type shared_communication_channel: SharedCommunicationChannel, optional
    """

    def __init__(
        self,
        communication_setting: SftpCommunicationOptionsCommunicationSetting = SENTINEL,
        sftp_get_options: SftpGetOptions = SENTINEL,
        sftp_send_options: SftpSendOptions = SENTINEL,
        sftp_settings: SftpSettings = SENTINEL,
        shared_communication_channel: SharedCommunicationChannel = SENTINEL,
        **kwargs,
    ):
        """SftpCommunicationOptions

        :param communication_setting: communication_setting, defaults to None
        :type communication_setting: SftpCommunicationOptionsCommunicationSetting, optional
        :param sftp_get_options: sftp_get_options, defaults to None
        :type sftp_get_options: SftpGetOptions, optional
        :param sftp_send_options: sftp_send_options, defaults to None
        :type sftp_send_options: SftpSendOptions, optional
        :param sftp_settings: sftp_settings, defaults to None
        :type sftp_settings: SftpSettings, optional
        :param shared_communication_channel: shared_communication_channel, defaults to None
        :type shared_communication_channel: SharedCommunicationChannel, optional
        """
        if communication_setting is not SENTINEL:
            self.communication_setting = self._enum_matching(
                communication_setting,
                SftpCommunicationOptionsCommunicationSetting.list(),
                "communication_setting",
            )
        if sftp_get_options is not SENTINEL:
            self.sftp_get_options = self._define_object(
                sftp_get_options, SftpGetOptions
            )
        if sftp_send_options is not SENTINEL:
            self.sftp_send_options = self._define_object(
                sftp_send_options, SftpSendOptions
            )
        if sftp_settings is not SENTINEL:
            self.sftp_settings = self._define_object(sftp_settings, SftpSettings)
        if shared_communication_channel is not SENTINEL:
            self.shared_communication_channel = self._define_object(
                shared_communication_channel, SharedCommunicationChannel
            )
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    AtomConnectionFieldExtensionSummaryQueryConfig,
    AtomConnectionFieldExtensionSummaryQueryResponse,
)


class AtomConnectionFieldExtensionSummaryService(BaseService):

    @cast_models
    def query_atom_connection_field_extension_summary(
        self, request_body: AtomConnectionFieldExtensionSummaryQueryConfig = None
    ) -> Union[AtomConnectionFieldExtensionSummaryQueryResponse, str]:
        """All filters are required except for `extensionGroupId`, which is required only for a multi-install integration pack. You can obtain valid values for each filter by using the QUERY operation on the Atom Extensions object.

         For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: AtomConnectionFieldExtensionSummaryQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AtomConnectionFieldExtensionSummaryQueryResponse, str]
        """

        Validator(
            AtomConnectionFieldExtensionSummaryQueryConfig
        ).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AtomConnectionFieldExtensionSummary/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AtomConnectionFieldExtensionSummaryQueryResponse._unmap(response)
        if content == "application/xml":
            return AtomConnectionFieldExtensionSummaryQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_atom_connection_field_extension_summary(
        self, request_body: str
    ) -> Union[AtomConnectionFieldExtensionSummaryQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AtomConnectionFieldExtensionSummaryQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AtomConnectionFieldExtensionSummary/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AtomConnectionFieldExtensionSummaryQueryResponse._unmap(response)
        if content == "application/xml":
            return AtomConnectionFieldExtensionSummaryQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..change_listener_status import ChangeListenerStatusService
from ...models import ChangeListenerStatusRequest


class ChangeListenerStatusServiceAsync(ChangeListenerStatusService):
    """
    Async Wrapper for ChangeListenerStatusServiceAsync
    """

    def create_change_listener_status(
        self, request_body: ChangeListenerStatusRequest = None
    ) -> Awaitable[None]:
        return to_async(super().create_change_listener_status)(request_body)




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..role import RoleService
from ...models import (
    Role,
    RoleBulkResponse,
    RoleBulkRequest,
    RoleQueryResponse,
    RoleQueryConfig,
)


class RoleServiceAsync(RoleService):
    """
    Async Wrapper for RoleServiceAsync
    """

    def create_role(self, request_body: Role = None) -> Awaitable[Union[Role, str]]:
        return to_async(super().create_role)(request_body)

    def get_role(self, id_: str) -> Awaitable[Union[Role, str]]:
        return to_async(super().get_role)(id_)

    def update_role(
        self, id_: str, request_body: Role = None
    ) -> Awaitable[Union[Role, str]]:
        return to_async(super().update_role)(id_, request_body)

    def delete_role(self, id_: str) -> Awaitable[None]:
        return to_async(super().delete_role)(id_)

    def bulk_role(
        self, request_body: RoleBulkRequest = None
    ) -> Awaitable[Union[RoleBulkResponse, str]]:
        return to_async(super().bulk_role)(request_body)

    def query_role(
        self, request_body: RoleQueryConfig = None
    ) -> Awaitable[Union[RoleQueryResponse, str]]:
        return to_async(super().query_role)(request_body)

    def query_more_role(
        self, request_body: str
    ) -> Awaitable[Union[RoleQueryResponse, str]]:
        return to_async(super().query_more_role)(request_body)




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .header import Header


@JsonMap({})
class HttpRequestHeaders(BaseModel):
    """HttpRequestHeaders

    :param header: header, defaults to None
    :type header: List[Header], optional
    """

    def __init__(self, header: List[Header] = SENTINEL, **kwargs):
        """HttpRequestHeaders

        :param header: header, defaults to None
        :type header: List[Header], optional
        """
        if header is not SENTINEL:
            self.header = self._define_list(header, Header)
        self._kwargs = kwargs




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class IntegrationPackInstallationType(Enum):
    """An enumeration representing different categories.

    :cvar SINGLE: "SINGLE"
    :vartype SINGLE: str
    :cvar MULTI: "MULTI"
    :vartype MULTI: str
    """

    SINGLE = "SINGLE"
    MULTI = "MULTI"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value, IntegrationPackInstallationType._member_map_.values()
            )
        )


@JsonMap(
    {"description": "Description", "id_": "id", "installation_type": "installationType"}
)
class IntegrationPack(BaseModel):
    """IntegrationPack

    :param description: A description of the integration pack.
    :type description: str
    :param id_: A unique ID assigned by the system to the integration pack., defaults to None
    :type id_: str, optional
    :param installation_type: The type of integration pack. Possible values:\<br /\>-   SINGLE — single-attach\<br /\>-   MULTI — multi-attach, defaults to None
    :type installation_type: IntegrationPackInstallationType, optional
    :param name: The name of the integration pack., defaults to None
    :type name: str, optional
    """

    def __init__(
        self,
        description: str,
        id_: str = SENTINEL,
        installation_type: IntegrationPackInstallationType = SENTINEL,
        name: str = SENTINEL,
        **kwargs
    ):
        """IntegrationPack

        :param description: A description of the integration pack.
        :type description: str
        :param id_: A unique ID assigned by the system to the integration pack., defaults to None
        :type id_: str, optional
        :param installation_type: The type of integration pack. Possible values:\<br /\>-   SINGLE — single-attach\<br /\>-   MULTI — multi-attach, defaults to None
        :type installation_type: IntegrationPackInstallationType, optional
        :param name: The name of the integration pack., defaults to None
        :type name: str, optional
        """
        self.description = description
        if id_ is not SENTINEL:
            self.id_ = id_
        if installation_type is not SENTINEL:
            self.installation_type = self._enum_matching(
                installation_type,
                IntegrationPackInstallationType.list(),
                "installation_type",
            )
        if name is not SENTINEL:
            self.name = name
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .environment_map_extension_external_component_simple_expression import (
    EnvironmentMapExtensionExternalComponentSimpleExpression,
)
from .environment_map_extension_external_component_grouping_expression import (
    EnvironmentMapExtensionExternalComponentGroupingExpression,
)


class EnvironmentMapExtensionExternalComponentExpressionGuard(OneOfBaseModel):
    class_list = {
        "EnvironmentMapExtensionExternalComponentSimpleExpression": EnvironmentMapExtensionExternalComponentSimpleExpression,
        "EnvironmentMapExtensionExternalComponentGroupingExpression": EnvironmentMapExtensionExternalComponentGroupingExpression,
    }


EnvironmentMapExtensionExternalComponentExpression = Union[
    EnvironmentMapExtensionExternalComponentSimpleExpression,
    EnvironmentMapExtensionExternalComponentGroupingExpression,
]




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class ObjectType(Enum):
    """An enumeration representing different categories.

    :cvar CLOUD: "Cloud"
    :vartype CLOUD: str
    :cvar CONNECTOR: "Connector"
    :vartype CONNECTOR: str
    :cvar DATAHUBMODEL: "Data Hub Model"
    :vartype DATAHUBMODEL: str
    :cvar INTEGRATIONPACK: "Integration Pack"
    :vartype INTEGRATIONPACK: str
    :cvar PUBLISHEDPROCESS: "Published Process"
    :vartype PUBLISHEDPROCESS: str
    :cvar ROLE: "Role"
    :vartype ROLE: str
    """

    CLOUD = "Cloud"
    CONNECTOR = "Connector"
    DATAHUBMODEL = "Data Hub Model"
    INTEGRATIONPACK = "Integration Pack"
    PUBLISHEDPROCESS = "Published Process"
    ROLE = "Role"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, ObjectType._member_map_.values()))


@JsonMap(
    {
        "object_type": "objectType",
        "resource_id": "resourceId",
        "resource_name": "resourceName",
    }
)
class Resource(BaseModel):
    """Resource

    :param object_type: Resource object type details., defaults to None
    :type object_type: ObjectType, optional
    :param resource_id: Account group resource ID.
    :type resource_id: str
    :param resource_name: Account group resource name.
    :type resource_name: str
    """

    def __init__(
        self,
        resource_id: str,
        resource_name: str,
        object_type: ObjectType = SENTINEL,
        **kwargs
    ):
        """Resource

        :param object_type: Resource object type details., defaults to None
        :type object_type: ObjectType, optional
        :param resource_id: Account group resource ID.
        :type resource_id: str
        :param resource_name: Account group resource name.
        :type resource_name: str
        """
        if object_type is not SENTINEL:
            self.object_type = self._enum_matching(
                object_type, ObjectType.list(), "object_type"
            )
        self.resource_id = resource_id
        self.resource_name = resource_name
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "original_process_id": "originalProcessId",
        "wrapper_process_id": "wrapperProcessId",
    }
)
class ProcessId(BaseModel):
    """ProcessId

    :param name: The name of the process., defaults to None
    :type name: str, optional
    :param original_process_id: A unique ID assigned by the system when the process is created., defaults to None
    :type original_process_id: str, optional
    :param wrapper_process_id: A unique ID assigned to each process associated with multi-install integration packs. A `wrapperProcessId` is generated when an IntegrationPackInstance is installed or created.   \>**Note:** The `wrapperProcessId` will not be returned for the single-install integration pack while making API calls, as it is only generated for the multi-install integration packs., defaults to None
    :type wrapper_process_id: str, optional
    """

    def __init__(
        self,
        name: str = SENTINEL,
        original_process_id: str = SENTINEL,
        wrapper_process_id: str = SENTINEL,
        **kwargs
    ):
        """ProcessId

        :param name: The name of the process., defaults to None
        :type name: str, optional
        :param original_process_id: A unique ID assigned by the system when the process is created., defaults to None
        :type original_process_id: str, optional
        :param wrapper_process_id: A unique ID assigned to each process associated with multi-install integration packs. A `wrapperProcessId` is generated when an IntegrationPackInstance is installed or created.   \>**Note:** The `wrapperProcessId` will not be returned for the single-install integration pack while making API calls, as it is only generated for the multi-install integration packs., defaults to None
        :type wrapper_process_id: str, optional
        """
        if name is not SENTINEL:
            self.name = name
        if original_process_id is not SENTINEL:
            self.original_process_id = original_process_id
        if wrapper_process_id is not SENTINEL:
            self.wrapper_process_id = wrapper_process_id
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..component_metadata import ComponentMetadataService
from ...models import (
    ComponentMetadata,
    ComponentMetadataBulkResponse,
    ComponentMetadataBulkRequest,
    ComponentMetadataQueryResponse,
    ComponentMetadataQueryConfig,
)


class ComponentMetadataServiceAsync(ComponentMetadataService):
    """
    Async Wrapper for ComponentMetadataServiceAsync
    """

    def create_component_metadata(
        self, request_body: ComponentMetadata = None
    ) -> Awaitable[Union[ComponentMetadata, str]]:
        return to_async(super().create_component_metadata)(request_body)

    def get_component_metadata(
        self, id_: str
    ) -> Awaitable[Union[ComponentMetadata, str]]:
        return to_async(super().get_component_metadata)(id_)

    def update_component_metadata(
        self, id_: str, request_body: ComponentMetadata = None
    ) -> Awaitable[Union[ComponentMetadata, str]]:
        return to_async(super().update_component_metadata)(id_, request_body)

    def delete_component_metadata(self, id_: str) -> Awaitable[None]:
        return to_async(super().delete_component_metadata)(id_)

    def bulk_component_metadata(
        self, request_body: ComponentMetadataBulkRequest = None
    ) -> Awaitable[Union[ComponentMetadataBulkResponse, str]]:
        return to_async(super().bulk_component_metadata)(request_body)

    def query_component_metadata(
        self, request_body: ComponentMetadataQueryConfig = None
    ) -> Awaitable[Union[ComponentMetadataQueryResponse, str]]:
        return to_async(super().query_component_metadata)(request_body)

    def query_more_component_metadata(
        self, request_body: str
    ) -> Awaitable[Union[ComponentMetadataQueryResponse, str]]:
        return to_async(super().query_more_component_metadata)(request_body)




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"split_length": "splitLength"})
class MapExtensionsStringSplit(BaseModel):
    """MapExtensionsStringSplit

    :param delimiter: delimiter, defaults to None
    :type delimiter: str, optional
    :param split_length: split_length, defaults to None
    :type split_length: int, optional
    """

    def __init__(
        self, delimiter: str = SENTINEL, split_length: int = SENTINEL, **kwargs
    ):
        """MapExtensionsStringSplit

        :param delimiter: delimiter, defaults to None
        :type delimiter: str, optional
        :param split_length: split_length, defaults to None
        :type split_length: int, optional
        """
        if delimiter is not SENTINEL:
            self.delimiter = delimiter
        if split_length is not SENTINEL:
            self.split_length = split_length
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"previous_value": "previousValue"})
class AuditLogProperty(BaseModel):
    """AuditLogProperty

    :param name: name, defaults to None
    :type name: str, optional
    :param previous_value: previous_value, defaults to None
    :type previous_value: str, optional
    :param value: value, defaults to None
    :type value: str, optional
    """

    def __init__(
        self,
        name: str = SENTINEL,
        previous_value: str = SENTINEL,
        value: str = SENTINEL,
        **kwargs
    ):
        """AuditLogProperty

        :param name: name, defaults to None
        :type name: str, optional
        :param previous_value: previous_value, defaults to None
        :type previous_value: str, optional
        :param value: value, defaults to None
        :type value: str, optional
        """
        if name is not SENTINEL:
            self.name = name
        if previous_value is not SENTINEL:
            self.previous_value = previous_value
        if value is not SENTINEL:
            self.value = value
        self._kwargs = kwargs




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class ScheduleType(Enum):
    """An enumeration representing different categories.

    :cvar NEVER: "NEVER"
    :vartype NEVER: str
    :cvar FIRST: "FIRST"
    :vartype FIRST: str
    :cvar LAST: "LAST"
    :vartype LAST: str
    """

    NEVER = "NEVER"
    FIRST = "FIRST"
    LAST = "LAST"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, ScheduleType._member_map_.values()))


@JsonMap(
    {
        "atom_id": "atomId",
        "day_of_week": "dayOfWeek",
        "hour_of_day": "hourOfDay",
        "schedule_type": "scheduleType",
        "time_zone": "timeZone",
    }
)
class RuntimeReleaseSchedule(BaseModel):
    """RuntimeReleaseSchedule

    :param atom_id: The ID of the container for which you want to set a schedule.
    :type atom_id: str
    :param day_of_week: The day of the week that you want to receive updates on the Runtime, Runtime cluster, or Runtime cloud. \<br /\> 1. Required if scheduleType is set to FIRST or LAST
    :type day_of_week: str
    :param hour_of_day: The hour of the day that you want to receive updates on the Runtime, Runtime cluster, or Runtime cloud. 1. Must be between 0-23\<br /\> 2. Required if scheduleType is set to FIRST or LAST, defaults to None
    :type hour_of_day: int, optional
    :param schedule_type: Required. Determines whether you want to receive the updates when available, and if so, whether you receive them in the first or second \(last\) week they are available prior to the .-   FIRST - Update within the first week that updates are available\<br /\> 1. LAST - Update within the second week that updates are available\<br /\>2. NEVER - Update with the
    :type schedule_type: ScheduleType
    :param time_zone: The time zone of your set schedule. \<br /\>1. Must be a [valid time zone](/api/platformapi#section/Introduction/Valid-time-zones) \<br /\>2. Required if scheduleType is set to FIRST or LAST
    :type time_zone: str
    """

    def __init__(
        self,
        atom_id: str = SENTINEL,
        day_of_week: str = SENTINEL,
        schedule_type: ScheduleType = SENTINEL,
        time_zone: str = SENTINEL,
        hour_of_day: int = SENTINEL,
        **kwargs
    ):
        """RuntimeReleaseSchedule

        :param atom_id: The ID of the container for which you want to set a schedule.
        :type atom_id: str
        :param day_of_week: The day of the week that you want to receive updates on the Runtime, Runtime cluster, or Runtime cloud. \<br /\> 1. Required if scheduleType is set to FIRST or LAST
        :type day_of_week: str, optional
        :param hour_of_day: The hour of the day that you want to receive updates on the Runtime, Runtime cluster, or Runtime cloud. 1. Must be between 0-23\<br /\> 2. Required if scheduleType is set to FIRST or LAST, defaults to None
        :type hour_of_day: int, optional
        :param schedule_type: Required. Determines whether you want to receive the updates when available, and if so, whether you receive them in the first or second \(last\) week they are available prior to the .-   FIRST - Update within the first week that updates are available\<br /\> 1. LAST - Update within the second week that updates are available\<br /\>2. NEVER - Update with the
        :type schedule_type: ScheduleType
        :param time_zone: The time zone of your set schedule. \<br /\>1. Must be a [valid time zone](/api/platformapi#section/Introduction/Valid-time-zones) \<br /\>2. Required if scheduleType is set to FIRST or LAST
        :type time_zone: str, optional
        """
        if atom_id is not SENTINEL:
            self.atom_id = atom_id
        if day_of_week is not SENTINEL:
            self.day_of_week = day_of_week
        if hour_of_day is not SENTINEL:
            self.hour_of_day = hour_of_day
        if schedule_type is not SENTINEL:
            self.schedule_type = self._enum_matching(
                schedule_type, ScheduleType.list(), "schedule_type"
            )
        if time_zone is not SENTINEL:
            self.time_zone = time_zone
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .property_pair import PropertyPair


@JsonMap({})
class CustomProperties(BaseModel):
    """CustomProperties

    :param properties: properties, defaults to None
    :type properties: List[PropertyPair], optional
    """

    def __init__(self, properties: List[PropertyPair] = SENTINEL, **kwargs):
        """CustomProperties

        :param properties: properties, defaults to None
        :type properties: List[PropertyPair], optional
        """
        if properties is not SENTINEL:
            self.properties = self._define_list(properties, PropertyPair)
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .component_environment_attachment_simple_expression import (
    ComponentEnvironmentAttachmentSimpleExpression,
)
from .component_environment_attachment_grouping_expression import (
    ComponentEnvironmentAttachmentGroupingExpression,
)


class ComponentEnvironmentAttachmentExpressionGuard(OneOfBaseModel):
    class_list = {
        "ComponentEnvironmentAttachmentSimpleExpression": ComponentEnvironmentAttachmentSimpleExpression,
        "ComponentEnvironmentAttachmentGroupingExpression": ComponentEnvironmentAttachmentGroupingExpression,
    }


ComponentEnvironmentAttachmentExpression = Union[
    ComponentEnvironmentAttachmentSimpleExpression,
    ComponentEnvironmentAttachmentGroupingExpression,
]




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .cloud_attachment_secrets_manager_provider import CloudAttachmentSecretsManagerProvider


@JsonMap(
    {
        "container_id": "containerId",
        "secrets_manager_provider": "secretsManagerProvider",
    }
)
class CloudAttachmentSecretsConfigurationRequest(BaseModel):
    """CloudAttachmentSecretsConfigurationRequest

    :param container_id: container_id
    :type container_id: str
    :param secrets_manager_provider: secrets_manager_provider
    :type secrets_manager_provider: CloudAttachmentSecretsManagerProvider
    """

    def __init__(
        self,
        container_id: str,
        secrets_manager_provider: CloudAttachmentSecretsManagerProvider,
        **kwargs,
    ):
        """CloudAttachmentSecretsConfigurationRequest

        :param container_id: container_id
        :type container_id: str
        :param secrets_manager_provider: secrets_manager_provider
        :type secrets_manager_provider: CloudAttachmentSecretsManagerProvider
        """
        self.container_id = container_id
        self.secrets_manager_provider = self._define_object(
            secrets_manager_provider, CloudAttachmentSecretsManagerProvider
        )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .listener_port_configuration import ListenerPortConfiguration


@JsonMap(
    {
        "api_type": "apiType",
        "auth_type": "authType",
        "base_url": "baseUrl",
        "listener_ports": "listenerPorts",
    }
)
class SharedWebServerCloudTennantGeneral(BaseModel):
    """SharedWebServerCloudTennantGeneral

    :param api_type: api_type
    :type api_type: str
    :param auth_type: auth_type
    :type auth_type: str
    :param base_url: base_url
    :type base_url: str
    :param listener_ports: listener_ports
    :type listener_ports: ListenerPortConfiguration
    """

    def __init__(
        self,
        api_type: str,
        auth_type: str,
        base_url: str,
        listener_ports: ListenerPortConfiguration,
        **kwargs,
    ):
        """SharedWebServerCloudTennantGeneral

        :param api_type: api_type
        :type api_type: str
        :param auth_type: auth_type
        :type auth_type: str
        :param base_url: base_url
        :type base_url: str
        :param listener_ports: listener_ports
        :type listener_ports: ListenerPortConfiguration
        """
        self.api_type = api_type
        self.auth_type = auth_type
        self.base_url = base_url
        self.listener_ports = self._define_object(
            listener_ports, ListenerPortConfiguration
        )
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .process_atom_attachment_simple_expression import (
    ProcessAtomAttachmentSimpleExpression,
)
from .process_atom_attachment_grouping_expression import (
    ProcessAtomAttachmentGroupingExpression,
)


class ProcessAtomAttachmentExpressionGuard(OneOfBaseModel):
    class_list = {
        "ProcessAtomAttachmentSimpleExpression": ProcessAtomAttachmentSimpleExpression,
        "ProcessAtomAttachmentGroupingExpression": ProcessAtomAttachmentGroupingExpression,
    }


ProcessAtomAttachmentExpression = Union[
    ProcessAtomAttachmentSimpleExpression, ProcessAtomAttachmentGroupingExpression
]




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .trading_partner_component_simple_expression import (
    TradingPartnerComponentSimpleExpression,
)
from .trading_partner_component_grouping_expression import (
    TradingPartnerComponentGroupingExpression,
)


class TradingPartnerComponentExpressionGuard(OneOfBaseModel):
    class_list = {
        "TradingPartnerComponentSimpleExpression": TradingPartnerComponentSimpleExpression,
        "TradingPartnerComponentGroupingExpression": TradingPartnerComponentGroupingExpression,
    }


TradingPartnerComponentExpression = Union[
    TradingPartnerComponentSimpleExpression, TradingPartnerComponentGroupingExpression
]




from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .public_certificate import PublicCertificate


class GlobalUsageCode(Enum):
    """An enumeration representing different categories.

    :cvar TEST: "Test"
    :vartype TEST: str
    :cvar PRODUCTION: "Production"
    :vartype PRODUCTION: str
    """

    TEST = "Test"
    PRODUCTION = "Production"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, GlobalUsageCode._member_map_.values()))


class PartnerIdType(Enum):
    """An enumeration representing different categories.

    :cvar DUNS: "DUNS"
    :vartype DUNS: str
    """

    DUNS = "DUNS"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, PartnerIdType._member_map_.values()))


@JsonMap(
    {
        "encryption_public_certificate": "encryptionPublicCertificate",
        "global_partner_classification_code": "globalPartnerClassificationCode",
        "global_usage_code": "globalUsageCode",
        "partner_id": "partnerId",
        "partner_id_type": "partnerIdType",
        "partner_location": "partnerLocation",
        "signing_public_certificate": "signingPublicCertificate",
        "supply_chain_code": "supplyChainCode",
    }
)
class RosettaNetControlInfo(BaseModel):
    """RosettaNetControlInfo

    :param encryption_public_certificate: encryption_public_certificate, defaults to None
    :type encryption_public_certificate: PublicCertificate, optional
    :param global_partner_classification_code: global_partner_classification_code, defaults to None
    :type global_partner_classification_code: str, optional
    :param global_usage_code: global_usage_code, defaults to None
    :type global_usage_code: GlobalUsageCode, optional
    :param partner_id: partner_id, defaults to None
    :type partner_id: str, optional
    :param partner_id_type: partner_id_type, defaults to None
    :type partner_id_type: PartnerIdType, optional
    :param partner_location: partner_location, defaults to None
    :type partner_location: str, optional
    :param signing_public_certificate: signing_public_certificate, defaults to None
    :type signing_public_certificate: PublicCertificate, optional
    :param supply_chain_code: supply_chain_code, defaults to None
    :type supply_chain_code: str, optional
    """

    def __init__(
        self,
        encryption_public_certificate: PublicCertificate = SENTINEL,
        global_partner_classification_code: str = SENTINEL,
        global_usage_code: GlobalUsageCode = SENTINEL,
        partner_id: str = SENTINEL,
        partner_id_type: PartnerIdType = SENTINEL,
        partner_location: str = SENTINEL,
        signing_public_certificate: PublicCertificate = SENTINEL,
        supply_chain_code: str = SENTINEL,
        **kwargs,
    ):
        """RosettaNetControlInfo

        :param encryption_public_certificate: encryption_public_certificate, defaults to None
        :type encryption_public_certificate: PublicCertificate, optional
        :param global_partner_classification_code: global_partner_classification_code, defaults to None
        :type global_partner_classification_code: str, optional
        :param global_usage_code: global_usage_code, defaults to None
        :type global_usage_code: GlobalUsageCode, optional
        :param partner_id: partner_id, defaults to None
        :type partner_id: str, optional
        :param partner_id_type: partner_id_type, defaults to None
        :type partner_id_type: PartnerIdType, optional
        :param partner_location: partner_location, defaults to None
        :type partner_location: str, optional
        :param signing_public_certificate: signing_public_certificate, defaults to None
        :type signing_public_certificate: PublicCertificate, optional
        :param supply_chain_code: supply_chain_code, defaults to None
        :type supply_chain_code: str, optional
        """
        if encryption_public_certificate is not SENTINEL:
            self.encryption_public_certificate = self._define_object(
                encryption_public_certificate, PublicCertificate
            )
        if global_partner_classification_code is not SENTINEL:
            self.global_partner_classification_code = global_partner_classification_code
        if global_usage_code is not SENTINEL:
            self.global_usage_code = self._enum_matching(
                global_usage_code, GlobalUsageCode.list(), "global_usage_code"
            )
        if partner_id is not SENTINEL:
            self.partner_id = partner_id
        if partner_id_type is not SENTINEL:
            self.partner_id_type = self._enum_matching(
                partner_id_type, PartnerIdType.list(), "partner_id_type"
            )
        if partner_location is not SENTINEL:
            self.partner_location = partner_location
        if signing_public_certificate is not SENTINEL:
            self.signing_public_certificate = self._define_object(
                signing_public_certificate, PublicCertificate
            )
        if supply_chain_code is not SENTINEL:
            self.supply_chain_code = supply_chain_code
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .rosetta_net_connector_record_expression import (
    RosettaNetConnectorRecordExpression,
    RosettaNetConnectorRecordExpressionGuard,
)
from .rosetta_net_connector_record_simple_expression import (
    RosettaNetConnectorRecordSimpleExpression,
)
from .rosetta_net_connector_record_grouping_expression import (
    RosettaNetConnectorRecordGroupingExpression,
)


@JsonMap({})
class RosettaNetConnectorRecordQueryConfigQueryFilter(BaseModel):
    """RosettaNetConnectorRecordQueryConfigQueryFilter

    :param expression: expression
    :type expression: RosettaNetConnectorRecordExpression
    """

    def __init__(self, expression: RosettaNetConnectorRecordExpression, **kwargs):
        """RosettaNetConnectorRecordQueryConfigQueryFilter

        :param expression: expression
        :type expression: RosettaNetConnectorRecordExpression
        """
        self.expression = RosettaNetConnectorRecordExpressionGuard.return_one_of(
            expression
        )
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class RosettaNetConnectorRecordQueryConfig(BaseModel):
    """RosettaNetConnectorRecordQueryConfig

    :param query_filter: query_filter
    :type query_filter: RosettaNetConnectorRecordQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: RosettaNetConnectorRecordQueryConfigQueryFilter, **kwargs
    ):
        """RosettaNetConnectorRecordQueryConfig

        :param query_filter: query_filter
        :type query_filter: RosettaNetConnectorRecordQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, RosettaNetConnectorRecordQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .generic_connector_record_simple_expression import (
    GenericConnectorRecordSimpleExpression,
)
from .generic_connector_record_grouping_expression import (
    GenericConnectorRecordGroupingExpression,
)


class GenericConnectorRecordExpressionGuard(OneOfBaseModel):
    class_list = {
        "GenericConnectorRecordSimpleExpression": GenericConnectorRecordSimpleExpression,
        "GenericConnectorRecordGroupingExpression": GenericConnectorRecordGroupingExpression,
    }


GenericConnectorRecordExpression = Union[
    GenericConnectorRecordSimpleExpression, GenericConnectorRecordGroupingExpression
]




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import ExecutionRequest


class ExecutionRequestService(BaseService):

    @cast_models
    def create_execution_request(
        self, request_body: ExecutionRequest = None
    ) -> Union[ExecutionRequest, str]:
        """Submits the process to run and returns results immediately. The operation does not wait for the run to complete.

         - The Execution Request response returns a requestID, which you use to make a subsequent call to the [Execution Record object](/api/platformapi#tag/ExecutionRecord) to retrieve detailed information about the process run.
        - This operation returns an error when the client:
          -  Fails authentication or does not have the correct permissions
          -  Supplies an invalid Account ID
          -  Supplies an invalid Runtime ID
          -  Attempts to reach a deleted Atom
          -  Supplies an invalid Process ID
          -  Missing privileges to run processes on the given Runtime or its associated Environment.

        :param request_body: The request body., defaults to None
        :type request_body: ExecutionRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ExecutionRequest, str]
        """

        Validator(ExecutionRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/ExecutionRequest",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ExecutionRequest._unmap(response)
        if content == "application/xml":
            return ExecutionRequest._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .throughput_account_simple_expression import ThroughputAccountSimpleExpression
from .throughput_account_grouping_expression import ThroughputAccountGroupingExpression


class ThroughputAccountExpressionGuard(OneOfBaseModel):
    class_list = {
        "ThroughputAccountSimpleExpression": ThroughputAccountSimpleExpression,
        "ThroughputAccountGroupingExpression": ThroughputAccountGroupingExpression,
    }


ThroughputAccountExpression = Union[
    ThroughputAccountSimpleExpression, ThroughputAccountGroupingExpression
]




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..api_usage_count import ApiUsageCountService
from ...models import ApiUsageCountQueryResponse, ApiUsageCountQueryConfig


class ApiUsageCountServiceAsync(ApiUsageCountService):
    """
    Async Wrapper for ApiUsageCountServiceAsync
    """

    def query_api_usage_count(
        self, request_body: ApiUsageCountQueryConfig = None
    ) -> Awaitable[Union[ApiUsageCountQueryResponse, str]]:
        return to_async(super().query_api_usage_count)(request_body)

    def query_more_api_usage_count(
        self, request_body: str
    ) -> Awaitable[Union[ApiUsageCountQueryResponse, str]]:
        return to_async(super().query_more_api_usage_count)(request_body)




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .connector_field import ConnectorField


@JsonMap({"connector_field": "connectorField"})
class ConnectorFields(BaseModel):
    """Displays all connector-related fields from the connector included in this document.

    :param connector_field: connector_field, defaults to None
    :type connector_field: List[ConnectorField], optional
    """

    def __init__(self, connector_field: List[ConnectorField] = SENTINEL, **kwargs):
        """Displays all connector-related fields from the connector included in this document.

        :param connector_field: connector_field, defaults to None
        :type connector_field: List[ConnectorField], optional
        """
        if connector_field is not SENTINEL:
            self.connector_field = self._define_list(connector_field, ConnectorField)
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .observability_endpoint_authentication import ObservabilityEndpointAuthentication


@JsonMap({})
class ObservabilityEndpoint(BaseModel):
    """ObservabilityEndpoint

    :param authentication: authentication
    :type authentication: ObservabilityEndpointAuthentication
    :param url: url
    :type url: str
    """

    def __init__(
        self,
        authentication: ObservabilityEndpointAuthentication,
        url: str,
        **kwargs,
    ):
        """ObservabilityEndpoint

        :param authentication: authentication
        :type authentication: ObservabilityEndpointAuthentication
        :param url: url
        :type url: str
        """
        self.authentication = self._define_object(
            authentication, ObservabilityEndpointAuthentication
        )
        self.url = url
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .map_extension import MapExtension


@JsonMap(
    {
        "map": "Map",
        "environment_id": "environmentId",
        "extension_group_id": "extensionGroupId",
        "id_": "id",
        "map_id": "mapId",
        "process_id": "processId",
    }
)
class EnvironmentMapExtension(BaseModel):
    """EnvironmentMapExtension

    :param map: map, defaults to None
    :type map: MapExtension, optional
    :param environment_id: The ID of the environment., defaults to None
    :type environment_id: str, optional
    :param extension_group_id: The ID of the multi-install integration pack to which the extensible map applies, if applicable., defaults to None
    :type extension_group_id: str, optional
    :param id_: The ID of the object. This is a conceptual ID synthesized from the IDs of the Map, Process, Multi-install integration pack \(extensionGroupId\), and, if applicable Environment. After obtaining this value with a QUERY operation on the [Environment Map Extensions Summary object](/api/platformapi#tag/EnvironmentMapExtensionsSummary), you can retrieve or update the extensible map by specifying the ID in a GET or UPDATE operation on this object, defaults to None
    :type id_: str, optional
    :param map_id: The ID of the extensible map., defaults to None
    :type map_id: str, optional
    :param name: The name of the extensible map. This variable includes the source object definition name and the destination object definition name, separated by a hyphen., defaults to None
    :type name: str, optional
    :param process_id: The ID of the process., defaults to None
    :type process_id: str, optional
    """

    def __init__(
        self,
        map: MapExtension = SENTINEL,
        environment_id: str = SENTINEL,
        extension_group_id: str = SENTINEL,
        id_: str = SENTINEL,
        map_id: str = SENTINEL,
        name: str = SENTINEL,
        process_id: str = SENTINEL,
        **kwargs,
    ):
        """EnvironmentMapExtension

        :param map: map, defaults to None
        :type map: MapExtension, optional
        :param environment_id: The ID of the environment., defaults to None
        :type environment_id: str, optional
        :param extension_group_id: The ID of the multi-install integration pack to which the extensible map applies, if applicable., defaults to None
        :type extension_group_id: str, optional
        :param id_: The ID of the object. This is a conceptual ID synthesized from the IDs of the Map, Process, Multi-install integration pack \(extensionGroupId\), and, if applicable Environment. After obtaining this value with a QUERY operation on the [Environment Map Extensions Summary object](/api/platformapi#tag/EnvironmentMapExtensionsSummary), you can retrieve or update the extensible map by specifying the ID in a GET or UPDATE operation on this object, defaults to None
        :type id_: str, optional
        :param map_id: The ID of the extensible map., defaults to None
        :type map_id: str, optional
        :param name: The name of the extensible map. This variable includes the source object definition name and the destination object definition name, separated by a hyphen., defaults to None
        :type name: str, optional
        :param process_id: The ID of the process., defaults to None
        :type process_id: str, optional
        """
        if map is not SENTINEL:
            self.map = self._define_object(map, MapExtension)
        if environment_id is not SENTINEL:
            self.environment_id = environment_id
        if extension_group_id is not SENTINEL:
            self.extension_group_id = extension_group_id
        if id_ is not SENTINEL:
            self.id_ = id_
        if map_id is not SENTINEL:
            self.map_id = map_id
        if name is not SENTINEL:
            self.name = name
        if process_id is not SENTINEL:
            self.process_id = process_id
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..java_upgrade import JavaUpgradeService
from ...models import JavaUpgrade


class JavaUpgradeServiceAsync(JavaUpgradeService):
    """
    Async Wrapper for JavaUpgradeServiceAsync
    """

    def create_java_upgrade(
        self, request_body: JavaUpgrade = None
    ) -> Awaitable[Union[JavaUpgrade, str]]:
        return to_async(super().create_java_upgrade)(request_body)




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .role_reference import RoleReference


@JsonMap({"role_reference": "RoleReference"})
class PermittedRoles(BaseModel):
    """Optional. The defined role assigned to the available folder object.

    :param role_reference: role_reference, defaults to None
    :type role_reference: List[RoleReference], optional
    """

    def __init__(self, role_reference: List[RoleReference] = SENTINEL, **kwargs):
        """Optional. The defined role assigned to the available folder object.

        :param role_reference: role_reference, defaults to None
        :type role_reference: List[RoleReference], optional
        """
        if role_reference is not SENTINEL:
            self.role_reference = self._define_list(role_reference, RoleReference)
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({})
class DefaultOftpPartnerSendSettings(BaseModel):
    """DefaultOftpPartnerSendSettings

    :param cd: cd, defaults to None
    :type cd: bool, optional
    :param operation: operation, defaults to None
    :type operation: str, optional
    :param sfiddesc: sfiddesc, defaults to None
    :type sfiddesc: str, optional
    :param sfiddsn: sfiddsn, defaults to None
    :type sfiddsn: str, optional
    """

    def __init__(
        self,
        cd: bool = SENTINEL,
        operation: str = SENTINEL,
        sfiddesc: str = SENTINEL,
        sfiddsn: str = SENTINEL,
        **kwargs
    ):
        """DefaultOftpPartnerSendSettings

        :param cd: cd, defaults to None
        :type cd: bool, optional
        :param operation: operation, defaults to None
        :type operation: str, optional
        :param sfiddesc: sfiddesc, defaults to None
        :type sfiddesc: str, optional
        :param sfiddsn: sfiddsn, defaults to None
        :type sfiddsn: str, optional
        """
        if cd is not SENTINEL:
            self.cd = cd
        if operation is not SENTINEL:
            self.operation = operation
        if sfiddesc is not SENTINEL:
            self.sfiddesc = sfiddesc
        if sfiddsn is not SENTINEL:
            self.sfiddsn = sfiddsn
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"trust_server_cert": "trustServerCert"})
class HttpsslOptions(BaseModel):
    """HttpsslOptions

    :param clientauth: clientauth, defaults to None
    :type clientauth: bool, optional
    :param clientsslalias: clientsslalias, defaults to None
    :type clientsslalias: str, optional
    :param trust_server_cert: trust_server_cert, defaults to None
    :type trust_server_cert: bool, optional
    :param trustedcertalias: trustedcertalias, defaults to None
    :type trustedcertalias: str, optional
    """

    def __init__(
        self,
        clientauth: bool = SENTINEL,
        clientsslalias: str = SENTINEL,
        trust_server_cert: bool = SENTINEL,
        trustedcertalias: str = SENTINEL,
        **kwargs
    ):
        """HttpsslOptions

        :param clientauth: clientauth, defaults to None
        :type clientauth: bool, optional
        :param clientsslalias: clientsslalias, defaults to None
        :type clientsslalias: str, optional
        :param trust_server_cert: trust_server_cert, defaults to None
        :type trust_server_cert: bool, optional
        :param trustedcertalias: trustedcertalias, defaults to None
        :type trustedcertalias: str, optional
        """
        if clientauth is not SENTINEL:
            self.clientauth = clientauth
        if clientsslalias is not SENTINEL:
            self.clientsslalias = clientsslalias
        if trust_server_cert is not SENTINEL:
            self.trust_server_cert = trust_server_cert
        if trustedcertalias is not SENTINEL:
            self.trustedcertalias = trustedcertalias
        self._kwargs = kwargs




from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class TradacomsConnectorRecordSimpleExpressionOperator(Enum):
    """An enumeration representing different categories.

    :cvar EQUALS: "EQUALS"
    :vartype EQUALS: str
    :cvar STARTSWITH: "STARTS_WITH"
    :vartype STARTSWITH: str
    :cvar BETWEEN: "BETWEEN"
    :vartype BETWEEN: str
    :cvar GREATERTHAN: "GREATER_THAN"
    :vartype GREATERTHAN: str
    :cvar GREATERTHANOREQUAL: "GREATER_THAN_OR_EQUAL"
    :vartype GREATERTHANOREQUAL: str
    :cvar LESSTHAN: "LESS_THAN"
    :vartype LESSTHAN: str
    :cvar LESSTHANOREQUAL: "LESS_THAN_OR_EQUAL"
    :vartype LESSTHANOREQUAL: str
    """

    EQUALS = "EQUALS"
    STARTSWITH = "STARTS_WITH"
    BETWEEN = "BETWEEN"
    GREATERTHAN = "GREATER_THAN"
    GREATERTHANOREQUAL = "GREATER_THAN_OR_EQUAL"
    LESSTHAN = "LESS_THAN"
    LESSTHANOREQUAL = "LESS_THAN_OR_EQUAL"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                TradacomsConnectorRecordSimpleExpressionOperator._member_map_.values(),
            )
        )


class TradacomsConnectorRecordSimpleExpressionProperty(Enum):
    """An enumeration representing different categories.

    :cvar EXECUTIONID: "executionId"
    :vartype EXECUTIONID: str
    :cvar ATOMID: "atomId"
    :vartype ATOMID: str
    :cvar DATEPROCESSED: "dateProcessed"
    :vartype DATEPROCESSED: str
    :cvar ID: "id"
    :vartype ID: str
    :cvar ACTIONTYPE: "actionType"
    :vartype ACTIONTYPE: str
    :cvar CONNECTORTYPE: "connectorType"
    :vartype CONNECTORTYPE: str
    :cvar CONNECTORNAME: "connectorName"
    :vartype CONNECTORNAME: str
    :cvar OPERATIONNAME: "operationName"
    :vartype OPERATIONNAME: str
    :cvar DOCUMENTINDEX: "documentIndex"
    :vartype DOCUMENTINDEX: str
    :cvar SUCCESSFUL: "successful"
    :vartype SUCCESSFUL: str
    :cvar SIZE: "size"
    :vartype SIZE: str
    :cvar ERRORMESSAGE: "errorMessage"
    :vartype ERRORMESSAGE: str
    :cvar VALIDATIONSTATUS: "validationStatus"
    :vartype VALIDATIONSTATUS: str
    :cvar VALIDATIONREPORT: "validationReport"
    :vartype VALIDATIONREPORT: str
    :cvar SENDERNAME: "senderName"
    :vartype SENDERNAME: str
    :cvar RECEIVERNAME: "receiverName"
    :vartype RECEIVERNAME: str
    :cvar MESSAGETYPE: "messageType"
    :vartype MESSAGETYPE: str
    :cvar DATE: "date"
    :vartype DATE: str
    :cvar TIME: "time"
    :vartype TIME: str
    :cvar SENDERTRANSMISSIONREFERENCE: "senderTransmissionReference"
    :vartype SENDERTRANSMISSIONREFERENCE: str
    :cvar RECEIVERTRANSMISSIONREFERENCE: "receiverTransmissionReference"
    :vartype RECEIVERTRANSMISSIONREFERENCE: str
    :cvar APPLICATIONREFERENCE: "applicationReference"
    :vartype APPLICATIONREFERENCE: str
    :cvar TRANSMISSIONPRIORITYCODE: "transmissionPriorityCode"
    :vartype TRANSMISSIONPRIORITYCODE: str
    :cvar FILEGENERATIONNUMBER: "fileGenerationNumber"
    :vartype FILEGENERATIONNUMBER: str
    :cvar FILEVERSIONNUMBER: "fileVersionNumber"
    :vartype FILEVERSIONNUMBER: str
    """

    EXECUTIONID = "executionId"
    ATOMID = "atomId"
    DATEPROCESSED = "dateProcessed"
    ID = "id"
    ACTIONTYPE = "actionType"
    CONNECTORTYPE = "connectorType"
    CONNECTORNAME = "connectorName"
    OPERATIONNAME = "operationName"
    DOCUMENTINDEX = "documentIndex"
    SUCCESSFUL = "successful"
    SIZE = "size"
    ERRORMESSAGE = "errorMessage"
    VALIDATIONSTATUS = "validationStatus"
    VALIDATIONREPORT = "validationReport"
    SENDERNAME = "senderName"
    RECEIVERNAME = "receiverName"
    MESSAGETYPE = "messageType"
    DATE = "date"
    TIME = "time"
    SENDERTRANSMISSIONREFERENCE = "senderTransmissionReference"
    RECEIVERTRANSMISSIONREFERENCE = "receiverTransmissionReference"
    APPLICATIONREFERENCE = "applicationReference"
    TRANSMISSIONPRIORITYCODE = "transmissionPriorityCode"
    FILEGENERATIONNUMBER = "fileGenerationNumber"
    FILEVERSIONNUMBER = "fileVersionNumber"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                TradacomsConnectorRecordSimpleExpressionProperty._member_map_.values(),
            )
        )


@JsonMap({})
class TradacomsConnectorRecordSimpleExpression(BaseModel):
    """TradacomsConnectorRecordSimpleExpression

    :param argument: argument, defaults to None
    :type argument: List[str], optional
    :param operator: operator
    :type operator: TradacomsConnectorRecordSimpleExpressionOperator
    :param property: property
    :type property: TradacomsConnectorRecordSimpleExpressionProperty
    """

    def __init__(
        self,
        operator: TradacomsConnectorRecordSimpleExpressionOperator,
        property: TradacomsConnectorRecordSimpleExpressionProperty,
        argument: List[str] = SENTINEL,
        **kwargs
    ):
        """TradacomsConnectorRecordSimpleExpression

        :param argument: argument, defaults to None
        :type argument: List[str], optional
        :param operator: operator
        :type operator: TradacomsConnectorRecordSimpleExpressionOperator
        :param property: property
        :type property: TradacomsConnectorRecordSimpleExpressionProperty
        """
        if argument is not SENTINEL:
            self.argument = argument
        self.operator = self._enum_matching(
            operator,
            TradacomsConnectorRecordSimpleExpressionOperator.list(),
            "operator",
        )
        self.property = self._enum_matching(
            property,
            TradacomsConnectorRecordSimpleExpressionProperty.list(),
            "property",
        )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .connector_expression import ConnectorExpression, ConnectorExpressionGuard
from .connector_simple_expression import ConnectorSimpleExpression
from .connector_grouping_expression import ConnectorGroupingExpression


@JsonMap({})
class ConnectorQueryConfigQueryFilter(BaseModel):
    """ConnectorQueryConfigQueryFilter

    :param expression: expression
    :type expression: ConnectorExpression
    """

    def __init__(self, expression: ConnectorExpression, **kwargs):
        """ConnectorQueryConfigQueryFilter

        :param expression: expression
        :type expression: ConnectorExpression
        """
        self.expression = ConnectorExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class ConnectorQueryConfig(BaseModel):
    """ConnectorQueryConfig

    :param query_filter: query_filter
    :type query_filter: ConnectorQueryConfigQueryFilter
    """

    def __init__(self, query_filter: ConnectorQueryConfigQueryFilter, **kwargs):
        """ConnectorQueryConfig

        :param query_filter: query_filter
        :type query_filter: ConnectorQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, ConnectorQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .as2_communication_options import As2CommunicationOptions
from .disk_communication_options import DiskCommunicationOptions
from .ftp_communication_options import FtpCommunicationOptions
from .http_communication_options import HttpCommunicationOptions
from .mllp_communication_options import MllpCommunicationOptions
from .oftp_communication_options import OftpCommunicationOptions
from .sftp_communication_options import SftpCommunicationOptions


@JsonMap(
    {
        "as2_communication_options": "AS2CommunicationOptions",
        "disk_communication_options": "DiskCommunicationOptions",
        "ftp_communication_options": "FTPCommunicationOptions",
        "http_communication_options": "HTTPCommunicationOptions",
        "mllp_communication_options": "MLLPCommunicationOptions",
        "oftp_communication_options": "OFTPCommunicationOptions",
        "sftp_communication_options": "SFTPCommunicationOptions",
    }
)
class PartnerCommunication(BaseModel):
    """PartnerCommunication

    :param as2_communication_options: as2_communication_options, defaults to None
    :type as2_communication_options: As2CommunicationOptions, optional
    :param disk_communication_options: disk_communication_options, defaults to None
    :type disk_communication_options: DiskCommunicationOptions, optional
    :param ftp_communication_options: ftp_communication_options, defaults to None
    :type ftp_communication_options: FtpCommunicationOptions, optional
    :param http_communication_options: http_communication_options, defaults to None
    :type http_communication_options: HttpCommunicationOptions, optional
    :param mllp_communication_options: mllp_communication_options, defaults to None
    :type mllp_communication_options: MllpCommunicationOptions, optional
    :param oftp_communication_options: oftp_communication_options, defaults to None
    :type oftp_communication_options: OftpCommunicationOptions, optional
    :param sftp_communication_options: sftp_communication_options, defaults to None
    :type sftp_communication_options: SftpCommunicationOptions, optional
    """

    def __init__(
        self,
        as2_communication_options: As2CommunicationOptions = SENTINEL,
        disk_communication_options: DiskCommunicationOptions = SENTINEL,
        ftp_communication_options: FtpCommunicationOptions = SENTINEL,
        http_communication_options: HttpCommunicationOptions = SENTINEL,
        mllp_communication_options: MllpCommunicationOptions = SENTINEL,
        oftp_communication_options: OftpCommunicationOptions = SENTINEL,
        sftp_communication_options: SftpCommunicationOptions = SENTINEL,
        **kwargs,
    ):
        """PartnerCommunication

        :param as2_communication_options: as2_communication_options, defaults to None
        :type as2_communication_options: As2CommunicationOptions, optional
        :param disk_communication_options: disk_communication_options, defaults to None
        :type disk_communication_options: DiskCommunicationOptions, optional
        :param ftp_communication_options: ftp_communication_options, defaults to None
        :type ftp_communication_options: FtpCommunicationOptions, optional
        :param http_communication_options: http_communication_options, defaults to None
        :type http_communication_options: HttpCommunicationOptions, optional
        :param mllp_communication_options: mllp_communication_options, defaults to None
        :type mllp_communication_options: MllpCommunicationOptions, optional
        :param oftp_communication_options: oftp_communication_options, defaults to None
        :type oftp_communication_options: OftpCommunicationOptions, optional
        :param sftp_communication_options: sftp_communication_options, defaults to None
        :type sftp_communication_options: SftpCommunicationOptions, optional
        """
        if as2_communication_options is not SENTINEL:
            self.as2_communication_options = self._define_object(
                as2_communication_options, As2CommunicationOptions
            )
        if disk_communication_options is not SENTINEL:
            self.disk_communication_options = self._define_object(
                disk_communication_options, DiskCommunicationOptions
            )
        if ftp_communication_options is not SENTINEL:
            self.ftp_communication_options = self._define_object(
                ftp_communication_options, FtpCommunicationOptions
            )
        if http_communication_options is not SENTINEL:
            self.http_communication_options = self._define_object(
                http_communication_options, HttpCommunicationOptions
            )
        if mllp_communication_options is not SENTINEL:
            self.mllp_communication_options = self._define_object(
                mllp_communication_options, MllpCommunicationOptions
            )
        if oftp_communication_options is not SENTINEL:
            self.oftp_communication_options = self._define_object(
                oftp_communication_options, OftpCommunicationOptions
            )
        if sftp_communication_options is not SENTINEL:
            self.sftp_communication_options = self._define_object(
                sftp_communication_options, SftpCommunicationOptions
            )
        self._kwargs = kwargs

    def _map(self):
        """
        Convert to dict for API operations.

        Uses passthrough for most protocols, stripping only read-only keys
        (CommunicationSetting, SharedCommunicationChannel) that the API rejects on UPDATE.
        FTP/SFTP use whitelists with port/maxFileCount int conversions.
        """
        result = {}

        # Helper to extract minimal settings for each protocol
        def extract_ftp_settings(ftp_opts):
            if not ftp_opts:
                return None
            mapped = ftp_opts._map() if hasattr(ftp_opts, '_map') else ftp_opts
            settings = mapped.get('FTPSettings', {})
            # Keep only essential fields, ensuring port is integer
            port = settings.get('port')
            if port is not None:
                port = int(port)
            result = {
                'FTPSettings': {
                    'host': settings.get('host'),
                    'port': port,
                    'user': settings.get('user'),
                    'password': settings.get('password', ''),
                    'connectionMode': settings.get('connectionMode', 'passive')
                }
            }
            # Include SSL options if present
            if 'FTPSSLOptions' in settings:
                result['FTPSettings']['FTPSSLOptions'] = settings['FTPSSLOptions']
            # Include Get/Send options if present (for remote_directory and action preservation)
            if 'FTPGetOptions' in mapped:
                get_opts = mapped['FTPGetOptions'].copy()
                # Ensure maxFileCount is int (API returns string, but requires int on update)
                if 'maxFileCount' in get_opts and get_opts['maxFileCount'] is not None:
                    get_opts['maxFileCount'] = int(get_opts['maxFileCount'])
                result['FTPGetOptions'] = get_opts
            if 'FTPSendOptions' in mapped:
                result['FTPSendOptions'] = mapped['FTPSendOptions']
            return result

        def extract_sftp_settings(sftp_opts):
            if not sftp_opts:
                return None
            mapped = sftp_opts._map() if hasattr(sftp_opts, '_map') else sftp_opts
            settings = mapped.get('SFTPSettings', {})
            # Ensure port is integer
            port = settings.get('port')
            if port is not None:
                port = int(port)
            result = {
                'SFTPSettings': {
                    'host': settings.get('host'),
                    'port': port,
                    'user': settings.get('user'),
                    'password': settings.get('password', '')
                }
            }
            # Include SSH options if present
            if 'SFTPSSHOptions' in settings:
                result['SFTPSettings']['SFTPSSHOptions'] = settings['SFTPSSHOptions']
            # Include Get/Send options if present (for remote_directory preservation)
            if 'SFTPGetOptions' in mapped:
                get_opts = mapped['SFTPGetOptions'].copy()
                # Ensure maxFileCount is int (API returns string, but requires int on update)
                if 'maxFileCount' in get_opts and get_opts['maxFileCount'] is not None:
                    get_opts['maxFileCount'] = int(get_opts['maxFileCount'])
                result['SFTPGetOptions'] = get_opts
            if 'SFTPSendOptions' in mapped:
                result['SFTPSendOptions'] = mapped['SFTPSendOptions']
            return result

        def extract_http_settings(http_opts):
            if not http_opts:
                return None
            mapped = http_opts._map() if hasattr(http_opts, '_map') else http_opts
            # Strip read-only keys that the API rejects on UPDATE
            mapped.pop('CommunicationSetting', None)
            mapped.pop('SharedCommunicationChannel', None)
            return mapped if mapped else None

        def extract_disk_settings(disk_opts):
            if not disk_opts:
                return None
            mapped = disk_opts._map() if hasattr(disk_opts, '_map') else disk_opts
            mapped.pop('CommunicationSetting', None)
            mapped.pop('SharedCommunicationChannel', None)
            # Ensure maxFileCount is int (API returns string, requires int on update)
            if 'DiskGetOptions' in mapped:
                get_opts = mapped['DiskGetOptions']
                if 'maxFileCount' in get_opts and get_opts['maxFileCount'] is not None:
                    get_opts['maxFileCount'] = int(get_opts['maxFileCount'])
            return mapped if mapped else None

        def extract_as2_settings(as2_opts):
            if not as2_opts:
                return None
            mapped = as2_opts._map() if hasattr(as2_opts, '_map') else as2_opts
            # Strip read-only keys that the API rejects on UPDATE
            mapped.pop('CommunicationSetting', None)
            mapped.pop('SharedCommunicationChannel', None)
            return mapped if mapped else None

        # Extract each protocol if present
        if hasattr(self, 'ftp_communication_options'):
            ftp = extract_ftp_settings(self.ftp_communication_options)
            if ftp:
                result['FTPCommunicationOptions'] = ftp

        if hasattr(self, 'sftp_communication_options'):
            sftp = extract_sftp_settings(self.sftp_communication_options)
            if sftp:
                result['SFTPCommunicationOptions'] = sftp

        if hasattr(self, 'http_communication_options'):
            http = extract_http_settings(self.http_communication_options)
            if http:
                result['HTTPCommunicationOptions'] = http

        if hasattr(self, 'disk_communication_options'):
            disk = extract_disk_settings(self.disk_communication_options)
            if disk:
                result['DiskCommunicationOptions'] = disk

        if hasattr(self, 'as2_communication_options'):
            as2 = extract_as2_settings(self.as2_communication_options)
            if as2:
                result['AS2CommunicationOptions'] = as2

        # MLLP - pass through directly
        if hasattr(self, 'mllp_communication_options') and self.mllp_communication_options:
            mllp = self.mllp_communication_options
            mapped = mllp._map() if hasattr(mllp, '_map') else mllp
            if mapped:
                result['MLLPCommunicationOptions'] = mapped

        # OFTP - pass through, stripping read-only keys
        if hasattr(self, 'oftp_communication_options') and self.oftp_communication_options:
            oftp = self.oftp_communication_options
            mapped = oftp._map() if hasattr(oftp, '_map') else oftp
            if mapped:
                mapped.pop('CommunicationSetting', None)
                mapped.pop('SharedCommunicationChannel', None)
                result['OFTPCommunicationOptions'] = mapped

        return result




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .environment_atom_attachment_expression import (
    EnvironmentAtomAttachmentExpression,
    EnvironmentAtomAttachmentExpressionGuard,
)
from .environment_atom_attachment_simple_expression import (
    EnvironmentAtomAttachmentSimpleExpression,
)
from .environment_atom_attachment_grouping_expression import (
    EnvironmentAtomAttachmentGroupingExpression,
)


@JsonMap({})
class EnvironmentAtomAttachmentQueryConfigQueryFilter(BaseModel):
    """EnvironmentAtomAttachmentQueryConfigQueryFilter

    :param expression: expression
    :type expression: EnvironmentAtomAttachmentExpression
    """

    def __init__(self, expression: EnvironmentAtomAttachmentExpression, **kwargs):
        """EnvironmentAtomAttachmentQueryConfigQueryFilter

        :param expression: expression
        :type expression: EnvironmentAtomAttachmentExpression
        """
        self.expression = EnvironmentAtomAttachmentExpressionGuard.return_one_of(
            expression
        )
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class EnvironmentAtomAttachmentQueryConfig(BaseModel):
    """EnvironmentAtomAttachmentQueryConfig

    :param query_filter: query_filter
    :type query_filter: EnvironmentAtomAttachmentQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: EnvironmentAtomAttachmentQueryConfigQueryFilter, **kwargs
    ):
        """EnvironmentAtomAttachmentQueryConfig

        :param query_filter: query_filter
        :type query_filter: EnvironmentAtomAttachmentQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, EnvironmentAtomAttachmentQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .document import Document


@JsonMap({"document": "Document"})
class SelectedDocuments(BaseModel):
    """You must include either the `AllDocuments` field or the `SelectedDocuments` field in a request, but not both. Indicates that the Rerun Document operation reruns only those documents you specify in the `genericConnectorRecordId` value.

    :param document: document, defaults to None
    :type document: List[Document], optional
    """

    def __init__(self, document: List[Document] = SENTINEL, **kwargs):
        """You must include either the `AllDocuments` field or the `SelectedDocuments` field in a request, but not both. Indicates that the Rerun Document operation reruns only those documents you specify in the `genericConnectorRecordId` value.

        :param document: document, defaults to None
        :type document: List[Document], optional
        """
        if document is not SENTINEL:
            self.document = self._define_list(document, Document)
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.transport.utils import parse_xml_to_dict
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..models import (
    SharedCommunicationChannelComponent,
    SharedCommunicationChannelComponentBulkRequest,
    SharedCommunicationChannelComponentBulkResponse,
    SharedCommunicationChannelComponentQueryConfig,
    SharedCommunicationChannelComponentQueryResponse,
)


class SharedCommunicationChannelComponentService(BaseService):

    @cast_models
    def create_shared_communication_channel_component(
        self, request_body: SharedCommunicationChannelComponent = None
    ) -> Union[str, SharedCommunicationChannelComponent]:
        """The sample request creates a Shared Communication Component named `Disk Comms Channel`.

        :param request_body: The request body., defaults to None
        :type request_body: SharedCommunicationChannelComponent, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[str, SharedCommunicationChannelComponent]
        """

        Validator(SharedCommunicationChannelComponent).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/SharedCommunicationChannelComponent",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/xml":
            return SharedCommunicationChannelComponent._unmap(parse_xml_to_dict(response))
        if content == "application/json":
            return SharedCommunicationChannelComponent._unmap(response)
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def get_shared_communication_channel_component(
        self, id_: str
    ) -> Union[SharedCommunicationChannelComponent, str]:
        """Send an HTTP GET request where `{accountId}` is the ID of the authenticating account for the request and `{componentId}` is the ID of the component being retrieved.

        :param id_: ID of the component being retrieved.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[SharedCommunicationChannelComponent, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/SharedCommunicationChannelComponent/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return SharedCommunicationChannelComponent._unmap(response)
        if content == "application/xml":
            return SharedCommunicationChannelComponent._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def update_shared_communication_channel_component(
        self, id_: str, request_body: SharedCommunicationChannelComponent = None
    ) -> Union[SharedCommunicationChannelComponent, str]:
        """The sample request updates the component named `Disk Comms Channel`.

        :param request_body: The request body., defaults to None
        :type request_body: SharedCommunicationChannelComponent, optional
        :param id_: ID of the component that needs updating.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[SharedCommunicationChannelComponent, str]
        """

        Validator(SharedCommunicationChannelComponent).is_optional().validate(
            request_body
        )
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/SharedCommunicationChannelComponent/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return SharedCommunicationChannelComponent._unmap(response)
        if content == "application/xml":
            return SharedCommunicationChannelComponent._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_shared_communication_channel_component(self, id_: str) -> None:
        """If the Shared Communication Channel component is deleted successfully, the response is `true`.

        :param id_: ID of the component that you want to delete.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/SharedCommunicationChannelComponent/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)

    @cast_models
    def bulk_shared_communication_channel_component(
        self, request_body: SharedCommunicationChannelComponentBulkRequest = None
    ) -> Union[SharedCommunicationChannelComponentBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: SharedCommunicationChannelComponentBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[SharedCommunicationChannelComponentBulkResponse, str]
        """

        Validator(
            SharedCommunicationChannelComponentBulkRequest
        ).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/SharedCommunicationChannelComponent/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return SharedCommunicationChannelComponentBulkResponse._unmap(response)
        if content == "application/xml":
            return SharedCommunicationChannelComponentBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_shared_communication_channel_component(
        self, request_body: SharedCommunicationChannelComponentQueryConfig = None
    ) -> Union[SharedCommunicationChannelComponentQueryResponse, str]:
        """For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

         The sample request query returns the Shared Communication Channel components using the AS2 standard for the authenticating account.

         >**Note:** The name field in a QUERY filter represents the object's `componentName` field.

        :param request_body: The request body., defaults to None
        :type request_body: SharedCommunicationChannelComponentQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[SharedCommunicationChannelComponentQueryResponse, str]
        """

        Validator(
            SharedCommunicationChannelComponentQueryConfig
        ).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/SharedCommunicationChannelComponent/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return SharedCommunicationChannelComponentQueryResponse._unmap(response)
        if content == "application/xml":
            # For XML responses, parse the XML and convert to dict structure
            import xml.etree.ElementTree as ET
            try:
                root = ET.fromstring(response)
                # Extract query results from XML
                result_data = {
                    'numberOfResults': 0,
                    'result': []
                }

                # Get number of results if present
                num_results = root.get('numberOfResults')
                if num_results:
                    result_data['numberOfResults'] = int(num_results)

                # Get query token if present
                query_token = root.get('queryToken')
                if query_token:
                    result_data['queryToken'] = query_token

                # Parse each result element
                for item in root.findall('.//{http://api.platform.boomi.com/}result'):
                    channel_data = {}
                    # Get attributes
                    for attr, value in item.attrib.items():
                        channel_data[attr] = value
                    # Get child elements
                    for child in item:
                        tag = child.tag.replace('{http://api.platform.boomi.com/}', '')
                        channel_data[tag] = child.text
                    if channel_data:
                        result_data['result'].append(channel_data)

                result_data['numberOfResults'] = len(result_data['result'])
                return SharedCommunicationChannelComponentQueryResponse._unmap(result_data)
            except ET.ParseError:
                # If XML parsing fails, return empty result
                return SharedCommunicationChannelComponentQueryResponse._unmap({
                    'numberOfResults': 0,
                    'result': []
                })
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_shared_communication_channel_component(
        self, request_body: str
    ) -> Union[SharedCommunicationChannelComponentQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[SharedCommunicationChannelComponentQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/SharedCommunicationChannelComponent/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return SharedCommunicationChannelComponentQueryResponse._unmap(response)
        if content == "application/xml":
            # For XML responses, parse the XML and convert to dict structure
            import xml.etree.ElementTree as ET
            try:
                root = ET.fromstring(response)
                # Extract query results from XML
                result_data = {
                    'numberOfResults': 0,
                    'result': []
                }

                # Get number of results if present
                num_results = root.get('numberOfResults')
                if num_results:
                    result_data['numberOfResults'] = int(num_results)

                # Get query token if present
                query_token = root.get('queryToken')
                if query_token:
                    result_data['queryToken'] = query_token

                # Parse each result element
                for item in root.findall('.//{http://api.platform.boomi.com/}result'):
                    channel_data = {}
                    # Get attributes
                    for attr, value in item.attrib.items():
                        channel_data[attr] = value
                    # Get child elements
                    for child in item:
                        tag = child.tag.replace('{http://api.platform.boomi.com/}', '')
                        channel_data[tag] = child.text
                    if channel_data:
                        result_data['result'].append(channel_data)

                result_data['numberOfResults'] = len(result_data['result'])
                return SharedCommunicationChannelComponentQueryResponse._unmap(result_data)
            except ET.ParseError:
                # If XML parsing fails, return empty result
                return SharedCommunicationChannelComponentQueryResponse._unmap({
                    'numberOfResults': 0,
                    'result': []
                })
        raise ApiError("Error on deserializing the response.", status, response)




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    AccountCloudAttachmentProperties,
    AccountCloudAttachmentPropertiesAsyncResponse,
    AsyncOperationTokenResult,
)


class AccountCloudAttachmentPropertiesService(BaseService):

    @cast_models
    def update_account_cloud_attachment_properties(
        self, id_: str, request_body: AccountCloudAttachmentProperties = None
    ) -> Union[AccountCloudAttachmentProperties, str]:
        """Modifies one or more Account Cloud attachment properties.

         - To update property values, include all property names that you want to modify including their new values in the request body you do not need to provide the full list of properties in the request. This action is equivalent to editing property values on the [Attachment quotas tab](https://help.boomi.com/docs/Atomsphere/Integration/Integration%20management/r-atm-Attachment_Quotas_tab_4fbc3fff-7aaf-4bbd-a2dc-25d0edb5189c) (Manage > Cloud Management) in the user interface.

         - To ensure a successful request, you must provide valid property names and their accepted values in the request body. Otherwise, it returns an error.

         - The response returns a status code of 200 indicating a successful request. To verify updates made to a property, you can make a new GET request or view the Cloud attachment quotas tab (Manage > Cloud Management) in the user interface.

         - To modify properties, you must be the owner of the private Runtime cloud, and both the Runtime cloud and its attachments must be online.

        :param request_body: The request body., defaults to None
        :type request_body: AccountCloudAttachmentProperties, optional
        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountCloudAttachmentProperties, str]
        """

        Validator(AccountCloudAttachmentProperties).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountCloudAttachmentProperties/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountCloudAttachmentProperties._unmap(response)
        if content == "application/xml":
            return AccountCloudAttachmentProperties._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def async_get_account_cloud_attachment_properties(
        self, id_: str
    ) -> Union[AsyncOperationTokenResult, str]:
        """Use the GET operation to return and view a full list of Account Cloud attachment properties and their current values. This action is equivalent to viewing the [Attachment quotas](https://help.boomi.com/docs/Atomsphere/Integration/Integration%20management/r-atm-Attachment_Quotas_tab_4fbc3fff-7aaf-4bbd-a2dc-25d0edb5189c) tab (Manage > Cloud Management) in the user interface.
         >**Note:** The Cloud and attachments to which you are calling must be online. Cloud owners and users that own the Cloud attachments can use this operation.

        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AsyncOperationTokenResult, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/async/AccountCloudAttachmentProperties/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AsyncOperationTokenResult._unmap(response)
        if content == "application/xml":
            return AsyncOperationTokenResult._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def async_token_account_cloud_attachment_properties(
        self, token: str
    ) -> Union[AccountCloudAttachmentPropertiesAsyncResponse, str]:
        """Send a second GET request with the token returned in the first GET request. The object returns a list of existing property names and values for the given account and Cloud.
         >**Note:** The Cloud and attachments to which you are calling must be online. Cloud owners and users that own the Cloud attachments can use this operation.

        :param token: Takes in the token from a previous call to return a result.
        :type token: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountCloudAttachmentPropertiesAsyncResponse, str]
        """

        Validator(str).validate(token)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/async/AccountCloudAttachmentProperties/response/{{token}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("token", token)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountCloudAttachmentPropertiesAsyncResponse._unmap(response)
        if content == "application/xml":
            return AccountCloudAttachmentPropertiesAsyncResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .partner_document_type import PartnerDocumentType


@JsonMap({"partner_document_type": "PartnerDocumentType"})
class PartnerDocumentTypes(BaseModel):
    """PartnerDocumentTypes

    :param partner_document_type: partner_document_type, defaults to None
    :type partner_document_type: List[PartnerDocumentType], optional
    """

    def __init__(
        self, partner_document_type: List[PartnerDocumentType] = SENTINEL, **kwargs
    ):
        """PartnerDocumentTypes

        :param partner_document_type: partner_document_type, defaults to None
        :type partner_document_type: List[PartnerDocumentType], optional
        """
        if partner_document_type is not SENTINEL:
            self.partner_document_type = self._define_list(
                partner_document_type, PartnerDocumentType
            )
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"dlq": "DLQ", "queue_name": "QueueName", "subscriber_name": "SubscriberName"})
class QueueAttributes(BaseModel):
    """\(Required\) An instance of a generalized queue object indicating the queue from which to send or obtain the messages. Contains information describing the desired queue.

    :param dlq: (Required) true, or false. Allows the user to choose the regular or dead letter queue for deleting messages. The default is `false`, which deletes messages from the regular queue.
    :type dlq: bool
    :param queue_name: (Required) The name of the queue or topic. You can find this in the [List Queues](/api/platformapi#tag/ListQueues) action or in Queue Management.
    :type queue_name: str
    :param subscriber_name: (Optional. Use only for topic subscribers.) The name of the subscriber as it exists on the Runtime. You can find this by using the [List Queues](/api/platformapi#tag/ListQueues) action or by looking up the Component ID of the process associated with the subscriber.  \>**Note:** This field is not only the subscriber name shown on the Queue Management screen of the user interface., defaults to None
    :type subscriber_name: str, optional
    """

    def __init__(
        self, dlq: bool, queue_name: str, subscriber_name: str = SENTINEL, **kwargs
    ):
        """\(Required\) An instance of a generalized queue object indicating the queue from which to send or obtain the messages. Contains information describing the desired queue.

        :param dlq: (Required) true, or false. Allows the user to choose the regular or dead letter queue for deleting messages. The default is `false`, which deletes messages from the regular queue.
        :type dlq: bool
        :param queue_name: (Required) The name of the queue or topic. You can find this in the [List Queues](/api/platformapi#tag/ListQueues) action or in Queue Management.
        :type queue_name: str
        :param subscriber_name: (Optional. Use only for topic subscribers.) The name of the subscriber as it exists on the Runtime. You can find this by using the [List Queues](/api/platformapi#tag/ListQueues) action or by looking up the Component ID of the process associated with the subscriber.  \>**Note:** This field is not only the subscriber name shown on the Queue Management screen of the user interface., defaults to None
        :type subscriber_name: str, optional
        """
        self.dlq = dlq
        self.queue_name = queue_name
        if subscriber_name is not SENTINEL:
            self.subscriber_name = subscriber_name
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..account_cloud_attachment_properties_default import (
    AccountCloudAttachmentPropertiesDefaultService,
)
from ...models import (
    AccountCloudAttachmentPropertiesDefault,
    AsyncOperationTokenResult,
    AccountCloudAttachmentPropertiesDefaultAsyncResponse,
)


class AccountCloudAttachmentPropertiesDefaultServiceAsync(
    AccountCloudAttachmentPropertiesDefaultService
):
    """
    Async Wrapper for AccountCloudAttachmentPropertiesDefaultServiceAsync
    """

    def update_account_cloud_attachment_properties_default(
        self, id_: str, request_body: AccountCloudAttachmentPropertiesDefault = None
    ) -> Awaitable[Union[AccountCloudAttachmentPropertiesDefault, str]]:
        return to_async(super().update_account_cloud_attachment_properties_default)(
            id_, request_body
        )

    def async_get_account_cloud_attachment_properties_default(
        self, id_: str
    ) -> Awaitable[Union[AsyncOperationTokenResult, str]]:
        return to_async(super().async_get_account_cloud_attachment_properties_default)(
            id_
        )

    def async_token_account_cloud_attachment_properties_default(
        self, token: str
    ) -> Awaitable[Union[AccountCloudAttachmentPropertiesDefaultAsyncResponse, str]]:
        return to_async(
            super().async_token_account_cloud_attachment_properties_default
        )(token)




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .environment_atom_attachment_simple_expression import (
    EnvironmentAtomAttachmentSimpleExpression,
)
from .environment_atom_attachment_grouping_expression import (
    EnvironmentAtomAttachmentGroupingExpression,
)


class EnvironmentAtomAttachmentExpressionGuard(OneOfBaseModel):
    class_list = {
        "EnvironmentAtomAttachmentSimpleExpression": EnvironmentAtomAttachmentSimpleExpression,
        "EnvironmentAtomAttachmentGroupingExpression": EnvironmentAtomAttachmentGroupingExpression,
    }


EnvironmentAtomAttachmentExpression = Union[
    EnvironmentAtomAttachmentSimpleExpression,
    EnvironmentAtomAttachmentGroupingExpression,
]




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .atom_security_policy import AtomSecurityPolicy


@JsonMap({})
class AtomSecurityPoliciesType(BaseModel):
    """AtomSecurityPoliciesType

    :param policies: policies, defaults to None
    :type policies: List[AtomSecurityPolicy], optional
    """

    def __init__(self, policies: List[AtomSecurityPolicy] = SENTINEL, **kwargs):
        """AtomSecurityPoliciesType

        :param policies: policies, defaults to None
        :type policies: List[AtomSecurityPolicy], optional
        """
        if policies is not SENTINEL:
            self.policies = self._define_list(policies, AtomSecurityPolicy)
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .counter import Counter


@JsonMap({"atom_id": "atomId"})
class AtomCounters(BaseModel):
    """AtomCounters

    :param atom_id: A unique ID assigned by the system to the Runtime., defaults to None
    :type atom_id: str, optional
    :param counter: counter, defaults to None
    :type counter: List[Counter], optional
    """

    def __init__(
        self, atom_id: str = SENTINEL, counter: List[Counter] = SENTINEL, **kwargs
    ):
        """AtomCounters

        :param atom_id: A unique ID assigned by the system to the Runtime., defaults to None
        :type atom_id: str, optional
        :param counter: counter, defaults to None
        :type counter: List[Counter], optional
        """
        if atom_id is not SENTINEL:
            self.atom_id = atom_id
        if counter is not SENTINEL:
            self.counter = self._define_list(counter, Counter)
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .custom_fields import CustomFields


@JsonMap(
    {
        "account": "account",
        "action_type": "actionType",
        "atom_id": "atomId",
        "connector_name": "connectorName",
        "connector_type": "connectorType",
        "custom_fields": "customFields",
        "date_processed": "dateProcessed",
        "document_index": "documentIndex",
        "error_message": "errorMessage",
        "execution_id": "executionId",
        "from_trading_partner": "fromTradingPartner",
        "id_": "id",
        "operation_name": "operationName",
        "size": "size",
        "successful": "successful",
        "to_trading_partner": "toTradingPartner",
        "custom_standard_id": "customStandardId",
        "custom_standard_name": "customStandardName",
        "message_date": "messageDate",
        "message_id": "messageId",
        "message_time": "messageTime",
        "message_type": "messageType",
        "receiver_id": "receiverId",
        "receiver_id_qualifier": "receiverIdQualifier",
        "sender_id": "senderId",
        "sender_id_qualifier": "senderIdQualifier",
        "standard_id": "standardId",
    }
)
class EdiCustomConnectorRecord(BaseModel):
    """EdiCustomConnectorRecord

    :param account: The ID of the account that ran this record.
    :type account: str
    :param action_type: The type of action with which this record corresponds — *Send* for an outbound interchange, *Get* for an inbound interchange using the Disk, FTP, or SFTP communication method, or *Listen* for an inbound interchange using the AS2 or HTTP communication method.
    :type action_type: str
    :param atom_id: The ID of the Runtime that processed this record.
    :type atom_id: str
    :param connector_name: The value is *Trading Partner* for a Custom trading partner Send operation, or *Start* for a Custom trading partner Listen operation.
    :type connector_name: str
    :param connector_type: *edicustom* is the connector type for any record.
    :type connector_type: str
    :param custom_fields: custom_fields
    :type custom_fields: CustomFields
    :param date_processed: The processing date and time of this record. Format is yyyy-MM-dd'T'HH:mm:ss'Z' \(for example, 2019-09-14T15:32:00Z\).
    :type date_processed: str
    :param document_index: The numerical index of this record in the execution., defaults to None
    :type document_index: int, optional
    :param error_message: Any error message associated with this record. This field is omitted for a successful interchange.
    :type error_message: str
    :param execution_id: The ID of the run.
    :type execution_id: str
    :param from_trading_partner: The name of the sending trading partner component., defaults to None
    :type from_trading_partner: str, optional
    :param id_: The ID of this record.
    :type id_: str
    :param operation_name: The name of the operation component that processed the record.
    :type operation_name: str
    :param size: The size, in bytes, of the document that corresponds to this record., defaults to None
    :type size: int, optional
    :param successful: Whether the record is a success or error., defaults to None
    :type successful: bool, optional
    :param to_trading_partner: The name of the receiving trading partner component., defaults to None
    :type to_trading_partner: str, optional
    :param custom_standard_id: custom_standard_id, defaults to None
    :type custom_standard_id: str, optional
    :param custom_standard_name: custom_standard_name, defaults to None
    :type custom_standard_name: str, optional
    :param message_date: message_date, defaults to None
    :type message_date: str, optional
    :param message_id: message_id, defaults to None
    :type message_id: str, optional
    :param message_time: message_time, defaults to None
    :type message_time: str, optional
    :param message_type: message_type, defaults to None
    :type message_type: str, optional
    :param receiver_id: receiver_id, defaults to None
    :type receiver_id: str, optional
    :param receiver_id_qualifier: receiver_id_qualifier, defaults to None
    :type receiver_id_qualifier: str, optional
    :param sender_id: sender_id, defaults to None
    :type sender_id: str, optional
    :param sender_id_qualifier: sender_id_qualifier, defaults to None
    :type sender_id_qualifier: str, optional
    :param standard_id: standard_id, defaults to None
    :type standard_id: str, optional
    :param version: version, defaults to None
    :type version: str, optional
    """

    def __init__(
        self,
        account: str,
        action_type: str,
        atom_id: str,
        connector_name: str,
        connector_type: str,
        custom_fields: CustomFields,
        date_processed: str,
        error_message: str,
        execution_id: str,
        id_: str,
        operation_name: str,
        document_index: int = SENTINEL,
        from_trading_partner: str = SENTINEL,
        size: int = SENTINEL,
        successful: bool = SENTINEL,
        to_trading_partner: str = SENTINEL,
        custom_standard_id: str = SENTINEL,
        custom_standard_name: str = SENTINEL,
        message_date: str = SENTINEL,
        message_id: str = SENTINEL,
        message_time: str = SENTINEL,
        message_type: str = SENTINEL,
        receiver_id: str = SENTINEL,
        receiver_id_qualifier: str = SENTINEL,
        sender_id: str = SENTINEL,
        sender_id_qualifier: str = SENTINEL,
        standard_id: str = SENTINEL,
        version: str = SENTINEL,
        **kwargs,
    ):
        """EdiCustomConnectorRecord

        :param account: The ID of the account that ran this record.
        :type account: str
        :param action_type: The type of action with which this record corresponds — *Send* for an outbound interchange, *Get* for an inbound interchange using the Disk, FTP, or SFTP communication method, or *Listen* for an inbound interchange using the AS2 or HTTP communication method.
        :type action_type: str
        :param atom_id: The ID of the Runtime that processed this record.
        :type atom_id: str
        :param connector_name: The value is *Trading Partner* for a Custom trading partner Send operation, or *Start* for a Custom trading partner Listen operation.
        :type connector_name: str
        :param connector_type: *edicustom* is the connector type for any record.
        :type connector_type: str
        :param custom_fields: custom_fields
        :type custom_fields: CustomFields
        :param date_processed: The processing date and time of this record. Format is yyyy-MM-dd'T'HH:mm:ss'Z' \(for example, 2019-09-14T15:32:00Z\).
        :type date_processed: str
        :param document_index: The numerical index of this record in the execution., defaults to None
        :type document_index: int, optional
        :param error_message: Any error message associated with this record. This field is omitted for a successful interchange.
        :type error_message: str
        :param execution_id: The ID of the run.
        :type execution_id: str
        :param from_trading_partner: The name of the sending trading partner component., defaults to None
        :type from_trading_partner: str, optional
        :param id_: The ID of this record.
        :type id_: str
        :param operation_name: The name of the operation component that processed the record.
        :type operation_name: str
        :param size: The size, in bytes, of the document that corresponds to this record., defaults to None
        :type size: int, optional
        :param successful: Whether the record is a success or error., defaults to None
        :type successful: bool, optional
        :param to_trading_partner: The name of the receiving trading partner component., defaults to None
        :type to_trading_partner: str, optional
        :param custom_standard_id: custom_standard_id, defaults to None
        :type custom_standard_id: str, optional
        :param custom_standard_name: custom_standard_name, defaults to None
        :type custom_standard_name: str, optional
        :param message_date: message_date, defaults to None
        :type message_date: str, optional
        :param message_id: message_id, defaults to None
        :type message_id: str, optional
        :param message_time: message_time, defaults to None
        :type message_time: str, optional
        :param message_type: message_type, defaults to None
        :type message_type: str, optional
        :param receiver_id: receiver_id, defaults to None
        :type receiver_id: str, optional
        :param receiver_id_qualifier: receiver_id_qualifier, defaults to None
        :type receiver_id_qualifier: str, optional
        :param sender_id: sender_id, defaults to None
        :type sender_id: str, optional
        :param sender_id_qualifier: sender_id_qualifier, defaults to None
        :type sender_id_qualifier: str, optional
        :param standard_id: standard_id, defaults to None
        :type standard_id: str, optional
        :param version: version, defaults to None
        :type version: str, optional
        """
        self.account = account
        self.action_type = action_type
        self.atom_id = atom_id
        self.connector_name = connector_name
        self.connector_type = connector_type
        self.custom_fields = self._define_object(custom_fields, CustomFields)
        self.date_processed = date_processed
        if document_index is not SENTINEL:
            self.document_index = document_index
        self.error_message = error_message
        self.execution_id = execution_id
        if from_trading_partner is not SENTINEL:
            self.from_trading_partner = from_trading_partner
        self.id_ = id_
        self.operation_name = operation_name
        if size is not SENTINEL:
            self.size = size
        if successful is not SENTINEL:
            self.successful = successful
        if to_trading_partner is not SENTINEL:
            self.to_trading_partner = to_trading_partner
        if custom_standard_id is not SENTINEL:
            self.custom_standard_id = custom_standard_id
        if custom_standard_name is not SENTINEL:
            self.custom_standard_name = custom_standard_name
        if message_date is not SENTINEL:
            self.message_date = message_date
        if message_id is not SENTINEL:
            self.message_id = message_id
        if message_time is not SENTINEL:
            self.message_time = message_time
        if message_type is not SENTINEL:
            self.message_type = message_type
        if receiver_id is not SENTINEL:
            self.receiver_id = receiver_id
        if receiver_id_qualifier is not SENTINEL:
            self.receiver_id_qualifier = receiver_id_qualifier
        if sender_id is not SENTINEL:
            self.sender_id = sender_id
        if sender_id_qualifier is not SENTINEL:
            self.sender_id_qualifier = sender_id_qualifier
        if standard_id is not SENTINEL:
            self.standard_id = standard_id
        if version is not SENTINEL:
            self.version = version
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..account_user_federation import AccountUserFederationService
from ...models import (
    AccountUserFederation,
    AccountUserFederationQueryResponse,
    AccountUserFederationQueryConfig,
)


class AccountUserFederationServiceAsync(AccountUserFederationService):
    """
    Async Wrapper for AccountUserFederationServiceAsync
    """

    def create_account_user_federation(
        self, request_body: AccountUserFederation = None
    ) -> Awaitable[Union[AccountUserFederation, str]]:
        return to_async(super().create_account_user_federation)(request_body)

    def query_account_user_federation(
        self, request_body: AccountUserFederationQueryConfig = None
    ) -> Awaitable[Union[AccountUserFederationQueryResponse, str]]:
        return to_async(super().query_account_user_federation)(request_body)

    def query_more_account_user_federation(
        self, request_body: str
    ) -> Awaitable[Union[AccountUserFederationQueryResponse, str]]:
        return to_async(super().query_more_account_user_federation)(request_body)

    def update_account_user_federation(
        self, id_: str, request_body: AccountUserFederation = None
    ) -> Awaitable[Union[AccountUserFederation, str]]:
        return to_async(super().update_account_user_federation)(id_, request_body)

    def delete_account_user_federation(self, id_: str) -> Awaitable[None]:
        return to_async(super().delete_account_user_federation)(id_)




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..document_count_account_group import DocumentCountAccountGroupService
from ...models import (
    DocumentCountAccountGroupQueryResponse,
    DocumentCountAccountGroupQueryConfig,
)


class DocumentCountAccountGroupServiceAsync(DocumentCountAccountGroupService):
    """
    Async Wrapper for DocumentCountAccountGroupServiceAsync
    """

    def query_document_count_account_group(
        self, request_body: DocumentCountAccountGroupQueryConfig = None
    ) -> Awaitable[Union[DocumentCountAccountGroupQueryResponse, str]]:
        return to_async(super().query_document_count_account_group)(request_body)

    def query_more_document_count_account_group(
        self, request_body: str
    ) -> Awaitable[Union[DocumentCountAccountGroupQueryResponse, str]]:
        return to_async(super().query_more_document_count_account_group)(request_body)




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"atom_id": "atomId"})
class CloudAtom(BaseModel):
    """CloudAtom

    :param atom_id: atom_id, defaults to None
    :type atom_id: str, optional
    :param deleted: deleted, defaults to None
    :type deleted: bool, optional
    """

    def __init__(self, atom_id: str = SENTINEL, deleted: bool = SENTINEL, **kwargs):
        """CloudAtom

        :param atom_id: atom_id, defaults to None
        :type atom_id: str, optional
        :param deleted: deleted, defaults to None
        :type deleted: bool, optional
        """
        if atom_id is not SENTINEL:
            self.atom_id = atom_id
        if deleted is not SENTINEL:
            self.deleted = deleted
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..cloud import CloudService
from ...models import (
    Cloud,
    CloudBulkResponse,
    CloudBulkRequest,
    CloudQueryResponse,
    CloudQueryConfig,
)


class CloudServiceAsync(CloudService):
    """
    Async Wrapper for CloudServiceAsync
    """

    def get_cloud(self, id_: str) -> Awaitable[Union[Cloud, str]]:
        return to_async(super().get_cloud)(id_)

    def bulk_cloud(
        self, request_body: CloudBulkRequest = None
    ) -> Awaitable[Union[CloudBulkResponse, str]]:
        return to_async(super().bulk_cloud)(request_body)

    def query_cloud(
        self, request_body: CloudQueryConfig = None
    ) -> Awaitable[Union[CloudQueryResponse, str]]:
        return to_async(super().query_cloud)(request_body)

    def query_more_cloud(
        self, request_body: str
    ) -> Awaitable[Union[CloudQueryResponse, str]]:
        return to_async(super().query_more_cloud)(request_body)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .all_documents import AllDocuments
from .selected_documents import SelectedDocuments


@JsonMap(
    {
        "all_documents": "AllDocuments",
        "selected_documents": "SelectedDocuments",
        "original_execution_id": "originalExecutionId",
        "record_url": "recordUrl",
        "request_id": "requestId",
    }
)
class RerunDocument(BaseModel):
    """RerunDocument

    :param all_documents: You must include either the `AllDocuments` field or the `SelectedDocuments` field in a request, but not both.  Indicates that the Rerun Document operation reruns all documents in the original run. When using AllDocuments in a request, you must also specify a `documentStatus` value.
    :type all_documents: AllDocuments
    :param selected_documents: You must include either the `AllDocuments` field or the `SelectedDocuments` field in a request, but not both. Indicates that the Rerun Document operation reruns only those documents you specify in the `genericConnectorRecordId` value.
    :type selected_documents: SelectedDocuments
    :param original_execution_id: The ID of the original process run. You can obtain the `originalExecutionId` from the user interface from the Process Reporting page, selecting the Actions menu for a specific process run, and selecting View Extended Information from the list of options.
    :type original_execution_id: str
    :param record_url: (Response-only field) The ID of the process run. The initial CREATE response returns this field and uses it in the subsequent call to find the corresponding run record., defaults to None
    :type record_url: str, optional
    :param request_id: (Response-only field) The full endpoint URL used to make a second call to the Execution Record object. This URL is provided for your convenience in recordUrl field of the initial CREATE response., defaults to None
    :type request_id: str, optional
    """

    def __init__(
        self,
        all_documents: AllDocuments,
        selected_documents: SelectedDocuments,
        original_execution_id: str,
        record_url: str = SENTINEL,
        request_id: str = SENTINEL,
        **kwargs,
    ):
        """RerunDocument

        :param all_documents: You must include either the `AllDocuments` field or the `SelectedDocuments` field in a request, but not both.  Indicates that the Rerun Document operation reruns all documents in the original run. When using AllDocuments in a request, you must also specify a `documentStatus` value.
        :type all_documents: AllDocuments
        :param selected_documents: You must include either the `AllDocuments` field or the `SelectedDocuments` field in a request, but not both. Indicates that the Rerun Document operation reruns only those documents you specify in the `genericConnectorRecordId` value.
        :type selected_documents: SelectedDocuments
        :param original_execution_id: The ID of the original process run. You can obtain the `originalExecutionId` from the user interface from the Process Reporting page, selecting the Actions menu for a specific process run, and selecting View Extended Information from the list of options.
        :type original_execution_id: str
        :param record_url: (Response-only field) The ID of the process run. The initial CREATE response returns this field and uses it in the subsequent call to find the corresponding run record., defaults to None
        :type record_url: str, optional
        :param request_id: (Response-only field) The full endpoint URL used to make a second call to the Execution Record object. This URL is provided for your convenience in recordUrl field of the initial CREATE response., defaults to None
        :type request_id: str, optional
        """
        self.all_documents = self._define_object(all_documents, AllDocuments)
        self.selected_documents = self._define_object(
            selected_documents, SelectedDocuments
        )
        self.original_execution_id = original_execution_id
        if record_url is not SENTINEL:
            self.record_url = record_url
        if request_id is not SENTINEL:
            self.request_id = request_id
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .map_extensions_function_step import MapExtensionsFunctionStep


@JsonMap({"step": "Step"})
class MapExtensionsFunctionSteps(BaseModel):
    """Defines the individual function steps and the order in which they need to occur within the greater user-defined function. The following attributes are used: 1.`position` - represents the step's number order in the greater function. 2.`cacheType` - indicates the caching behavior of the individual function step. The allowed `cacheType` values are:1.`None` \(default, if omitted in the request\)— It does not use Map function caching. 2. `ByDocument` — Caches the map function’s input and output values for each processed document. 3. `ByMap` — Caches the map function’s input and output values for each processed map. -   `id` - represents the function step's ID in the format of "FUNCEXT--xxxxxxxxxx". 4. `type` - represents the type of function \(for example, "MathCeil" or "CustomScripting"\).<br /> Within the `Steps` element, you also need to define the following `input` and `output` variables for each function step:<br />1. `default` - Optional. Specifies the input value that the function uses if not provided by the user.<br />2. `name` - the user-defined name of the associated input or output.

     >**Note:** The user interface automatically uses the used function type as the step name, but you can use this API object to change function step names. 3. `key` - the number ID assigned to a function step. This key is used to map function steps together in the `Mappings` attribute.

    :param step: step, defaults to None
    :type step: List[MapExtensionsFunctionStep], optional
    """

    def __init__(self, step: List[MapExtensionsFunctionStep] = SENTINEL, **kwargs):
        """Defines the individual function steps and the order in which they need to occur within the greater user-defined function. The following attributes are used: 1.`position` - represents the step's number order in the greater function. 2.`cacheType` - indicates the caching behavior of the individual function step. The allowed `cacheType` values are:1.`None` \(default, if omitted in the request\)— It does not use Map function caching. 2. `ByDocument` — Caches the map function’s input and output values for each processed document. 3. `ByMap` — Caches the map function’s input and output values for each processed map. -   `id` - represents the function step's ID in the format of "FUNCEXT--xxxxxxxxxx". 4. `type` - represents the type of function \(for example, "MathCeil" or "CustomScripting"\).<br /> Within the `Steps` element, you also need to define the following `input` and `output` variables for each function step:<br />1. `default` - Optional. Specifies the input value that the function uses if not provided by the user.<br />2. `name` - the user-defined name of the associated input or output.

         >**Note:** The user interface automatically uses the used function type as the step name, but you can use this API object to change function step names. 3. `key` - the number ID assigned to a function step. This key is used to map function steps together in the `Mappings` attribute.

        :param step: step, defaults to None
        :type step: List[MapExtensionsFunctionStep], optional
        """
        if step is not SENTINEL:
            self.step = self._define_list(step, MapExtensionsFunctionStep)
        self._kwargs = kwargs




from __future__ import annotations
from typing import Optional
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .audit_log_expression import AuditLogExpression, AuditLogExpressionGuard
from .audit_log_simple_expression import AuditLogSimpleExpression
from .audit_log_grouping_expression import AuditLogGroupingExpression


@JsonMap({})
class AuditLogQueryConfigQueryFilter(BaseModel):
    """AuditLogQueryConfigQueryFilter

    :param expression: expression
    :type expression: AuditLogExpression
    """

    def __init__(self, expression: AuditLogExpression, **kwargs):
        """AuditLogQueryConfigQueryFilter

        :param expression: expression
        :type expression: AuditLogExpression
        """
        self.expression = AuditLogExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class AuditLogQueryConfig(BaseModel):
    """AuditLogQueryConfig

    :param query_filter: query_filter (optional)
    :type query_filter: Optional[AuditLogQueryConfigQueryFilter]
    """

    def __init__(self, query_filter: Optional[AuditLogQueryConfigQueryFilter] = None, **kwargs):
        """AuditLogQueryConfig

        :param query_filter: query_filter (optional)
        :type query_filter: Optional[AuditLogQueryConfigQueryFilter]
        """
        self.query_filter = self._define_object(
            query_filter, AuditLogQueryConfigQueryFilter
        ) if query_filter is not None else None
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .map_extensions_date_time import MapExtensionsDateTime
from .map_extensions_number import MapExtensionsNumber


@JsonMap(
    {
        "character": "Character",
        "date_time": "DateTime",
        "number": "Number",
        "enforce_unique": "enforceUnique",
        "field_length_validation": "fieldLengthValidation",
        "max_length": "maxLength",
        "min_length": "minLength",
    }
)
class MapExtensionsExtendedNode(BaseModel):
    """MapExtensionsExtendedNode

    :param character: character, defaults to None
    :type character: dict, optional
    :param date_time: date_time, defaults to None
    :type date_time: MapExtensionsDateTime, optional
    :param number: number, defaults to None
    :type number: MapExtensionsNumber, optional
    :param enforce_unique: enforce_unique, defaults to None
    :type enforce_unique: bool, optional
    :param field_length_validation: field_length_validation, defaults to None
    :type field_length_validation: bool, optional
    :param mandatory: mandatory, defaults to None
    :type mandatory: bool, optional
    :param max_length: max_length, defaults to None
    :type max_length: int, optional
    :param min_length: min_length, defaults to None
    :type min_length: int, optional
    :param name: name, defaults to None
    :type name: str, optional
    """

    def __init__(
        self,
        character: dict = SENTINEL,
        date_time: MapExtensionsDateTime = SENTINEL,
        number: MapExtensionsNumber = SENTINEL,
        enforce_unique: bool = SENTINEL,
        field_length_validation: bool = SENTINEL,
        mandatory: bool = SENTINEL,
        max_length: int = SENTINEL,
        min_length: int = SENTINEL,
        name: str = SENTINEL,
        **kwargs,
    ):
        """MapExtensionsExtendedNode

        :param character: character, defaults to None
        :type character: dict, optional
        :param date_time: date_time, defaults to None
        :type date_time: MapExtensionsDateTime, optional
        :param number: number, defaults to None
        :type number: MapExtensionsNumber, optional
        :param enforce_unique: enforce_unique, defaults to None
        :type enforce_unique: bool, optional
        :param field_length_validation: field_length_validation, defaults to None
        :type field_length_validation: bool, optional
        :param mandatory: mandatory, defaults to None
        :type mandatory: bool, optional
        :param max_length: max_length, defaults to None
        :type max_length: int, optional
        :param min_length: min_length, defaults to None
        :type min_length: int, optional
        :param name: name, defaults to None
        :type name: str, optional
        """
        if character is not SENTINEL:
            self.character = character
        if date_time is not SENTINEL:
            self.date_time = self._define_object(date_time, MapExtensionsDateTime)
        if number is not SENTINEL:
            self.number = self._define_object(number, MapExtensionsNumber)
        if enforce_unique is not SENTINEL:
            self.enforce_unique = enforce_unique
        if field_length_validation is not SENTINEL:
            self.field_length_validation = field_length_validation
        if mandatory is not SENTINEL:
            self.mandatory = mandatory
        if max_length is not SENTINEL:
            self.max_length = max_length
        if min_length is not SENTINEL:
            self.min_length = min_length
        if name is not SENTINEL:
            self.name = name
        self._kwargs = kwargs




from enum import Enum
from typing import Any, Dict

from ...net.headers.base_header import BaseHeader


class DefaultHeadersKeys(Enum):
    """
    An enumeration of default headers.

    :ivar str ACCESS_AUTH: The access token authentication header.
    :ivar str BASIC_AUTH: The basic authentication header.
    :ivar str API_KEY_AUTH: The API key authentication header.
    """

    ACCESS_AUTH = "access_token_auth"
    BASIC_AUTH = "basic_auth"
    API_KEY_AUTH = "api_key_auth"


class DefaultHeaders:
    """
    A class to manage default headers.

    :ivar Dict[str, BaseHeader] _default_headers: The default headers.
    """

    def __init__(self):
        self._default_headers: Dict[str, BaseHeader] = {}

    def set_header(self, key: DefaultHeadersKeys, value: BaseHeader) -> None:
        """
        Set a default header.

        :param DefaultHeadersKeys key: The key of the header to set.
        :param Any value: The value to set for the header.
        """
        self._default_headers[key.value] = value

    def get_header(self, key: DefaultHeadersKeys) -> BaseHeader:
        """
        Get a default header.

        :param DefaultHeadersKeys key: The key of the header to get.
        :return: The value of the header.
        :rtype: Any
        """
        return self._default_headers[key.value]

    def get_headers(self) -> list:
        """
        Get the default headers.

        :return: A list of the default headers.
        :rtype: list
        """
        return list(self._default_headers.values())




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({})
class MapExtensionsNode(BaseModel):
    """MapExtensionsNode

    :param name: name, defaults to None
    :type name: str, optional
    :param xpath: xpath, defaults to None
    :type xpath: str, optional
    """

    def __init__(self, name: str = SENTINEL, xpath: str = SENTINEL, **kwargs):
        """MapExtensionsNode

        :param name: name, defaults to None
        :type name: str, optional
        :param xpath: xpath, defaults to None
        :type xpath: str, optional
        """
        if name is not SENTINEL:
            self.name = name
        if xpath is not SENTINEL:
            self.xpath = xpath
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap(
    {
        "gcp_client_id": "gcpClientId",
        "gcp_client_secret": "gcpClientSecret",
        "gcp_refresh_token": "gcpRefreshToken",
    }
)
class CloudAttachmentGcpUserAccountConfig(BaseModel):
    """CloudAttachmentGcpUserAccountConfig

    :param gcp_client_id: gcp_client_id
    :type gcp_client_id: str
    :param gcp_client_secret: gcp_client_secret
    :type gcp_client_secret: str
    :param gcp_refresh_token: gcp_refresh_token
    :type gcp_refresh_token: str
    """

    def __init__(
        self,
        gcp_client_id: str,
        gcp_client_secret: str,
        gcp_refresh_token: str,
        **kwargs,
    ):
        """CloudAttachmentGcpUserAccountConfig

        :param gcp_client_id: gcp_client_id
        :type gcp_client_id: str
        :param gcp_client_secret: gcp_client_secret
        :type gcp_client_secret: str
        :param gcp_refresh_token: gcp_refresh_token
        :type gcp_refresh_token: str
        """
        self.gcp_client_id = gcp_client_id
        self.gcp_client_secret = gcp_client_secret
        self.gcp_refresh_token = gcp_refresh_token
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .cross_reference_rows import CrossReferenceRows


@JsonMap(
    {
        "cross_reference_rows": "CrossReferenceRows",
        "id_": "id",
        "override_values": "overrideValues",
    }
)
class CrossReference(BaseModel):
    """CrossReference

    :param cross_reference_rows: cross_reference_rows
    :type cross_reference_rows: CrossReferenceRows
    :param id_: id_, defaults to None
    :type id_: str, optional
    :param name: name, defaults to None
    :type name: str, optional
    :param override_values: override_values, defaults to None
    :type override_values: bool, optional
    """

    def __init__(
        self,
        cross_reference_rows: CrossReferenceRows,
        id_: str = SENTINEL,
        name: str = SENTINEL,
        override_values: bool = SENTINEL,
        **kwargs,
    ):
        """CrossReference

        :param cross_reference_rows: cross_reference_rows
        :type cross_reference_rows: CrossReferenceRows
        :param id_: id_, defaults to None
        :type id_: str, optional
        :param name: name, defaults to None
        :type name: str, optional
        :param override_values: override_values, defaults to None
        :type override_values: bool, optional
        """
        self.cross_reference_rows = self._define_object(
            cross_reference_rows, CrossReferenceRows
        )
        if id_ is not SENTINEL:
            self.id_ = id_
        if name is not SENTINEL:
            self.name = name
        if override_values is not SENTINEL:
            self.override_values = override_values
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .trading_partner_processing_group_expression import (
    TradingPartnerProcessingGroupExpression,
    TradingPartnerProcessingGroupExpressionGuard,
)
from .trading_partner_processing_group_simple_expression import (
    TradingPartnerProcessingGroupSimpleExpression,
)
from .trading_partner_processing_group_grouping_expression import (
    TradingPartnerProcessingGroupGroupingExpression,
)


@JsonMap({})
class TradingPartnerProcessingGroupQueryConfigQueryFilter(BaseModel):
    """TradingPartnerProcessingGroupQueryConfigQueryFilter

    :param expression: expression
    :type expression: TradingPartnerProcessingGroupExpression
    """

    def __init__(self, expression: TradingPartnerProcessingGroupExpression, **kwargs):
        """TradingPartnerProcessingGroupQueryConfigQueryFilter

        :param expression: expression
        :type expression: TradingPartnerProcessingGroupExpression
        """
        self.expression = TradingPartnerProcessingGroupExpressionGuard.return_one_of(
            expression
        )
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class TradingPartnerProcessingGroupQueryConfig(BaseModel):
    """TradingPartnerProcessingGroupQueryConfig

    :param query_filter: query_filter
    :type query_filter: TradingPartnerProcessingGroupQueryConfigQueryFilter
    """

    def __init__(
        self,
        query_filter: TradingPartnerProcessingGroupQueryConfigQueryFilter,
        **kwargs,
    ):
        """TradingPartnerProcessingGroupQueryConfig

        :param query_filter: query_filter
        :type query_filter: TradingPartnerProcessingGroupQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, TradingPartnerProcessingGroupQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .oftp2_connector_record_expression import (
    Oftp2ConnectorRecordExpression,
    Oftp2ConnectorRecordExpressionGuard,
)
from .oftp2_connector_record_simple_expression import (
    Oftp2ConnectorRecordSimpleExpression,
)
from .oftp2_connector_record_grouping_expression import (
    Oftp2ConnectorRecordGroupingExpression,
)


@JsonMap({})
class Oftp2ConnectorRecordQueryConfigQueryFilter(BaseModel):
    """Oftp2ConnectorRecordQueryConfigQueryFilter

    :param expression: expression
    :type expression: Oftp2ConnectorRecordExpression
    """

    def __init__(self, expression: Oftp2ConnectorRecordExpression, **kwargs):
        """Oftp2ConnectorRecordQueryConfigQueryFilter

        :param expression: expression
        :type expression: Oftp2ConnectorRecordExpression
        """
        self.expression = Oftp2ConnectorRecordExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class Oftp2ConnectorRecordQueryConfig(BaseModel):
    """Oftp2ConnectorRecordQueryConfig

    :param query_filter: query_filter
    :type query_filter: Oftp2ConnectorRecordQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: Oftp2ConnectorRecordQueryConfigQueryFilter, **kwargs
    ):
        """Oftp2ConnectorRecordQueryConfig

        :param query_filter: query_filter
        :type query_filter: Oftp2ConnectorRecordQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, Oftp2ConnectorRecordQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .folder_expression import FolderExpression, FolderExpressionGuard
from .folder_simple_expression import FolderSimpleExpression
from .folder_grouping_expression import FolderGroupingExpression


@JsonMap({})
class FolderQueryConfigQueryFilter(BaseModel):
    """FolderQueryConfigQueryFilter

    :param expression: expression
    :type expression: FolderExpression
    """

    def __init__(self, expression: FolderExpression, **kwargs):
        """FolderQueryConfigQueryFilter

        :param expression: expression
        :type expression: FolderExpression
        """
        self.expression = FolderExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class FolderQueryConfig(BaseModel):
    """FolderQueryConfig

    :param query_filter: query_filter
    :type query_filter: FolderQueryConfigQueryFilter
    """

    def __init__(self, query_filter: FolderQueryConfigQueryFilter, **kwargs):
        """FolderQueryConfig

        :param query_filter: query_filter
        :type query_filter: FolderQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, FolderQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .stx_control_info import StxControlInfo


@JsonMap({"stx_control_info": "STXControlInfo"})
class TradacomsControlInfo(BaseModel):
    """TradacomsControlInfo

    :param stx_control_info: stx_control_info, defaults to None
    :type stx_control_info: StxControlInfo, optional
    """

    def __init__(self, stx_control_info: StxControlInfo = SENTINEL, **kwargs):
        """TradacomsControlInfo

        :param stx_control_info: stx_control_info, defaults to None
        :type stx_control_info: StxControlInfo, optional
        """
        if stx_control_info is not SENTINEL:
            self.stx_control_info = self._define_object(
                stx_control_info, StxControlInfo
            )
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "client_ssl_alias": "clientSSLAlias",
        "ssl_alias": "sslAlias",
        "use_client_ssl": "useClientSSL",
        "use_ssl": "useSSL",
    }
)
class MllpsslOptions(BaseModel):
    """MllpsslOptions

    :param client_ssl_alias: client_ssl_alias, defaults to None
    :type client_ssl_alias: str, optional
    :param ssl_alias: ssl_alias, defaults to None
    :type ssl_alias: str, optional
    :param use_client_ssl: use_client_ssl, defaults to None
    :type use_client_ssl: bool, optional
    :param use_ssl: use_ssl, defaults to None
    :type use_ssl: bool, optional
    """

    def __init__(
        self,
        client_ssl_alias: str = SENTINEL,
        ssl_alias: str = SENTINEL,
        use_client_ssl: bool = SENTINEL,
        use_ssl: bool = SENTINEL,
        **kwargs
    ):
        """MllpsslOptions

        :param client_ssl_alias: client_ssl_alias, defaults to None
        :type client_ssl_alias: str, optional
        :param ssl_alias: ssl_alias, defaults to None
        :type ssl_alias: str, optional
        :param use_client_ssl: use_client_ssl, defaults to None
        :type use_client_ssl: bool, optional
        :param use_ssl: use_ssl, defaults to None
        :type use_ssl: bool, optional
        """
        if client_ssl_alias is not SENTINEL:
            self.client_ssl_alias = client_ssl_alias
        if ssl_alias is not SENTINEL:
            self.ssl_alias = ssl_alias
        if use_client_ssl is not SENTINEL:
            self.use_client_ssl = use_client_ssl
        if use_ssl is not SENTINEL:
            self.use_ssl = use_ssl
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"atom_id": "atomId"})
class JavaRollback(BaseModel):
    """JavaRollback

    :param atom_id: The unique ID assigned by the system to the container. 1. Use the Runtime ID for Runtimes, Runtime clusters, and Runtime clouds found in the user interface by navigating to **Manage** \> **Runtime Management** and viewing the Runtime Information panel for a selected container. 2. Use the Gateway ID found in the user interface by navigating to **Configure Server** \> **Gateways** \> `\<gatewayName\>` \> Gateway Information panel. 3. Use the Broker ID found in the user interface by navigating to **Configure Server** \> **Authentication** \> `\<brokerName\>` \> Broker Information panel., defaults to None
    :type atom_id: str, optional
    """

    def __init__(self, atom_id: str = SENTINEL, **kwargs):
        """JavaRollback

        :param atom_id: The unique ID assigned by the system to the container. 1. Use the Runtime ID for Runtimes, Runtime clusters, and Runtime clouds found in the user interface by navigating to **Manage** \> **Runtime Management** and viewing the Runtime Information panel for a selected container. 2. Use the Gateway ID found in the user interface by navigating to **Configure Server** \> **Gateways** \> `\<gatewayName\>` \> Gateway Information panel. 3. Use the Broker ID found in the user interface by navigating to **Configure Server** \> **Authentication** \> `\<brokerName\>` \> Broker Information panel., defaults to None
        :type atom_id: str, optional
        """
        if atom_id is not SENTINEL:
            self.atom_id = atom_id
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .map_extension_browse_settings import MapExtensionBrowseSettings
from .map_extensions_profile import MapExtensionsProfile
from .map_extension_extend_profile import MapExtensionExtendProfile
from .map_extensions_functions import MapExtensionsFunctions
from .map_extensions_extended_mappings import MapExtensionsExtendedMappings


@JsonMap(
    {
        "browse_settings": "BrowseSettings",
        "destination_profile": "DestinationProfile",
        "destination_profile_extensions": "DestinationProfileExtensions",
        "extended_functions": "ExtendedFunctions",
        "extended_mappings": "ExtendedMappings",
        "source_profile": "SourceProfile",
        "source_profile_extensions": "SourceProfileExtensions",
    }
)
class MapExtension(BaseModel):
    """MapExtension

    :param browse_settings: Represents the Connection information and is used to re-import newly-appended profile fields. This attribute is only applicable for certain application Connectors. To perform a re-import, the client must use the EXECUTE operation. For more information, see the Customizing profiles section later in this topic.  **Note:** `containerId` is a required field when using BrowseSettings.
    :type browse_settings: MapExtensionBrowseSettings
    :param destination_profile: Represents the fields in a source or destination profile of the given map, and specifies the section of the map in which to extend.   \>**Note:** You cannot add or create new fields in the object's request body. You must add new fields in the user interface of the underlying Map component or from a dynamic reimport. See the Customizing profiles section later in this topic. The following SourceProfile attributes define fields in the source profile: - componentId - represents the object definition extension ID. A GET request returns this value. - name - the user-defined field label(s) found in the source profile. - xpath - represents the field location in the source profile hierarchy.
    :type destination_profile: MapExtensionsProfile
    :param destination_profile_extensions: Contains the user-defined custom fields for the source or destination profile. Applicable only to Flat File profiles. Represents the field and value configuration settings manually specified in the source or destination profile's Data Element tab. For more information, refer to [Customizing Profiles](/docs/APIs/PlatformAPI/Customizing_profiles_environment_map_extension).
    :type destination_profile_extensions: MapExtensionExtendProfile
    :param extended_functions: The definition of Map function steps used in the Map component. For detailed information about how to define Map functions in a request or response, see the topic [Environment Map Extension functions](/docs/APIs/PlatformAPI/Environment_Map_Extension_functions). You can use the Extended Functions attribute to define the following extensible map functions (supports standard and user-defined function types): - User Defined - Connector - Lookup - Date - Numeric - String - Custom Scripting - Property
    :type extended_functions: MapExtensionsFunctions
    :param extended_mappings: Represents the field mappings between profiles, functions or both. You can use the following attributes: - fromXPath - represents the source profile's field path or the function's output key from which you are mapping. - toXPath - represents the destination profile's field path or the function's input key to which you are mapping. - toFunction - represents the function ID from which you are mapping. - fromFunction - represents the function ID to which you are mapping. To properly define each of these attributes, see the section [How to configure ExtendedMappings](/docs/APIs/PlatformAPI/Customizing_profiles_environment_map_extension#how-to-configure-extendedmappings)
    :type extended_mappings: MapExtensionsExtendedMappings
    :param source_profile: Represents the fields in a source or destination profile of the given map, and specifies the section of the map in which to extend.   \>**Note:** You cannot add or create new fields in the object's request body. You must add new fields in the user interface of the underlying Map component or from a dynamic reimport. See the Customizing profiles section later in this topic. The following SourceProfile attributes define fields in the source profile: - componentId - represents the object definition extension ID. A GET request returns this value. - name - the user-defined field label(s) found in the source profile. - xpath - represents the field location in the source profile hierarchy.
    :type source_profile: MapExtensionsProfile
    :param source_profile_extensions: Contains the user-defined custom fields for the source or destination profile. Applicable only to Flat File profiles. Represents the field and value configuration settings manually specified in the source or destination profile's Data Element tab. For more information, refer to [Customizing Profiles](/docs/APIs/PlatformAPI/Customizing_profiles_environment_map_extension).
    :type source_profile_extensions: MapExtensionExtendProfile
    """

    def __init__(
        self,
        browse_settings: MapExtensionBrowseSettings,
        destination_profile: MapExtensionsProfile,
        destination_profile_extensions: MapExtensionExtendProfile,
        extended_functions: MapExtensionsFunctions,
        extended_mappings: MapExtensionsExtendedMappings,
        source_profile: MapExtensionsProfile,
        source_profile_extensions: MapExtensionExtendProfile,
        **kwargs,
    ):
        """MapExtension

        :param browse_settings: Represents the Connection information and is used to re-import newly-appended profile fields. This attribute is only applicable for certain application Connectors. To perform a re-import, the client must use the EXECUTE operation. For more information, see the Customizing profiles section later in this topic.  **Note:** `containerId` is a required field when using BrowseSettings.
        :type browse_settings: MapExtensionBrowseSettings
        :param destination_profile: Represents the fields in a source or destination profile of the given map, and specifies the section of the map in which to extend.   \>**Note:** You cannot add or create new fields in the object's request body. You must add new fields in the user interface of the underlying Map component or from a dynamic reimport. See the Customizing profiles section later in this topic. The following SourceProfile attributes define fields in the source profile: - componentId - represents the object definition extension ID. A GET request returns this value. - name - the user-defined field label(s) found in the source profile. - xpath - represents the field location in the source profile hierarchy.
        :type destination_profile: MapExtensionsProfile
        :param destination_profile_extensions: Contains the user-defined custom fields for the source or destination profile. Applicable only to Flat File profiles. Represents the field and value configuration settings manually specified in the source or destination profile's Data Element tab. For more information, refer to [Customizing Profiles](/docs/APIs/PlatformAPI/Customizing_profiles_environment_map_extension).
        :type destination_profile_extensions: MapExtensionExtendProfile
        :param extended_functions: The definition of Map function steps used in the Map component. For detailed information about how to define Map functions in a request or response, see the topic [Environment Map Extension functions](/docs/APIs/PlatformAPI/Environment_Map_Extension_functions). You can use the Extended Functions attribute to define the following extensible map functions (supports standard and user-defined function types): - User Defined - Connector - Lookup - Date - Numeric - String - Custom Scripting - Property
        :type extended_functions: MapExtensionsFunctions
        :param extended_mappings: Represents the field mappings between profiles, functions or both. You can use the following attributes: - fromXPath - represents the source profile's field path or the function's output key from which you are mapping. - toXPath - represents the destination profile's field path or the function's input key to which you are mapping. - toFunction - represents the function ID from which you are mapping. - fromFunction - represents the function ID to which you are mapping. To properly define each of these attributes, see the section [How to configure ExtendedMappings](/docs/APIs/PlatformAPI/Customizing_profiles_environment_map_extension#how-to-configure-extendedmappings)
        :type extended_mappings: MapExtensionsExtendedMappings
        :param source_profile: Represents the fields in a source or destination profile of the given map, and specifies the section of the map in which to extend.   \>**Note:** You cannot add or create new fields in the object's request body. You must add new fields in the user interface of the underlying Map component or from a dynamic reimport. See the Customizing profiles section later in this topic. The following SourceProfile attributes define fields in the source profile: - componentId - represents the object definition extension ID. A GET request returns this value. - name - the user-defined field label(s) found in the source profile. - xpath - represents the field location in the source profile hierarchy.
        :type source_profile: MapExtensionsProfile
        :param source_profile_extensions: Contains the user-defined custom fields for the source or destination profile. Applicable only to Flat File profiles. Represents the field and value configuration settings manually specified in the source or destination profile's Data Element tab. For more information, refer to [Customizing Profiles](/docs/APIs/PlatformAPI/Customizing_profiles_environment_map_extension).
        :type source_profile_extensions: MapExtensionExtendProfile
        """
        self.browse_settings = self._define_object(
            browse_settings, MapExtensionBrowseSettings
        )
        self.destination_profile = self._define_object(
            destination_profile, MapExtensionsProfile
        )
        self.destination_profile_extensions = self._define_object(
            destination_profile_extensions, MapExtensionExtendProfile
        )
        self.extended_functions = self._define_object(
            extended_functions, MapExtensionsFunctions
        )
        self.extended_mappings = self._define_object(
            extended_mappings, MapExtensionsExtendedMappings
        )
        self.source_profile = self._define_object(source_profile, MapExtensionsProfile)
        self.source_profile_extensions = self._define_object(
            source_profile_extensions, MapExtensionExtendProfile
        )
        self._kwargs = kwargs
